"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QualifiedPipelines = void 0;
const uuid_1 = require("uuid");
const utility_1 = require("@jfrog-pipelines/utility");
const SyncRepository_1 = require("./acl/syncService/SyncRepository");
const StepRepository_1 = require("./acl/stepService/StepRepository");
class QualifiedPipelines {
    constructor(workerPayload) {
        this.workerPayload = workerPayload;
        this.triggerablePipelineStepIds = [];
        this.triggerablePipelineSteps = [];
        this.triggersByPipelineStepId = {};
        this.syncRespository = new SyncRepository_1.SyncRepository();
        this.stepRepository = new StepRepository_1.StepRepository();
    }
    async getCreateRunPayload() {
        const { stepIdsToCheck, inResourceConnections, trigVersionsByResourceId } = await this.getProjectResourceResourceVersionPipelineStepConnection();
        const versionCreatedBySteps = await this.getVersionCreatedBySteps(stepIdsToCheck);
        this.triggerablePipelineSteps = await this.getTriggerablePipelineSteps({
            versionCreatedBySteps,
            inResourceConnections,
            trigVersionsByResourceId
        });
        if (this.triggerablePipelineSteps.length === 0) {
            return [];
        }
        const createRunPayload = [];
        const pipelineStepIdsByPipelineId = (0, utility_1.groupBy)(this.triggerablePipelineSteps, 'pipelineId');
        for (const pipelineId in pipelineStepIdsByPipelineId) {
            createRunPayload.push({
                jFrogPipelinesSessionId: this.workerPayload.jFrogPipelinesSessionId ?? (0, uuid_1.v4)(),
                inPayload: this.workerPayload,
                stepList: pipelineStepIdsByPipelineId[pipelineId],
                pipelineId: Number.parseInt(pipelineId, 10),
                triggersByPipelineStepId: this.triggersByPipelineStepId
            });
        }
        return createRunPayload;
    }
    async getVersionCreatedBySteps(stepIdsToCheck) {
        if ((0, utility_1.isEmpty)(this.workerPayload.resourceVersionIds)) {
            return [];
        }
        if ((0, utility_1.isEmpty)(stepIdsToCheck)) {
            return [];
        }
        return this.stepRepository.getStepByStepIds(stepIdsToCheck);
    }
    async getPipelineStepsByIds(pipelineStepIds) {
        const pipelineSteps = await this.syncRespository.getPipelineStepsByPipelineStepIds(pipelineStepIds);
        return pipelineSteps;
    }
    async getQualifiedStepByPipelineStepIds() {
        let queryStepIds = [];
        if (this.workerPayload.pipelineStepIds) {
            queryStepIds = this.workerPayload.pipelineStepIds;
        }
        if (this.workerPayload.pipelineStepId) {
            queryStepIds = this.workerPayload.pipelineStepId;
        }
        return this.syncRespository.getQualifiedStepByPipelineStepIds(queryStepIds);
    }
    async getProjectResourceResourceVersionPipelineStepConnection() {
        if (!this.workerPayload.resourceVersionIds || (0, utility_1.isEmpty)(this.workerPayload.resourceVersionIds)) {
            return {
                stepIdsToCheck: [],
                trigVersionsByResourceId: {},
                resourceVersions: [],
                inResourceConnections: [],
                outResourceConnections: [],
                pipelines: [],
                skipPipelineIdIfWaiting: []
            };
        }
        const { stepIdsToCheck, qualifiedResourceVersionsByResourceId, resourceVersionList, inResourceConnections, outResourceConnections, pipelines, skipPipelineIdIfWaiting } = await this.syncRespository.getProjectResourceResourceVersionPipelineStepConnection(this.workerPayload.resourceVersionIds);
        return {
            stepIdsToCheck: stepIdsToCheck,
            trigVersionsByResourceId: qualifiedResourceVersionsByResourceId,
            resourceVersions: resourceVersionList,
            inResourceConnections: inResourceConnections,
            outResourceConnections: outResourceConnections,
            pipelines: pipelines,
            skipPipelineIdIfWaiting: skipPipelineIdIfWaiting
        };
    }
    async getTriggerablePipelineSteps(options) {
        const { versionCreatedBySteps, inResourceConnections, trigVersionsByResourceId } = options;
        this.triggerablePipelineStepIds = await this.getQualifiedStepByPipelineStepIds();
        const isEmptyResourceVersionIds = (0, utility_1.isEmpty)(this.workerPayload.resourceVersionIds);
        if (isEmptyResourceVersionIds) {
            return [];
        }
        const triggersByPipelineStepId = {};
        if ((0, utility_1.isEmpty)(versionCreatedBySteps)) {
            this.triggerablePipelineStepIds = inResourceConnections.map((connection) => connection.pipelineStepId);
            for (const connection of inResourceConnections) {
                const [triggeringVersion] = [...trigVersionsByResourceId[connection.operationResourceId]];
                triggersByPipelineStepId[connection.pipelineStepId] = {
                    resourceVersionId: triggeringVersion.id,
                    resourceId: triggeringVersion.resourceId
                };
            }
        }
        else {
        }
        this.triggerablePipelineSteps = await this.getPipelineStepsByIds(this.triggerablePipelineStepIds);
        const disabledPipelineIds = [];
        const filteredTriggerablePipelineSteps = this.triggerablePipelineSteps.filter((pipelineStep) => !disabledPipelineIds.includes(pipelineStep.pipelineId));
        this.triggerablePipelineSteps = filteredTriggerablePipelineSteps;
        return this.triggerablePipelineSteps;
    }
}
exports.QualifiedPipelines = QualifiedPipelines;
//# sourceMappingURL=QualifiedPipelines.js.map