"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunWorker = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const errors_1 = require("@jfrog-pipelines/errors");
const CreateRunCommand_1 = require("./createRun/CreateRunCommand");
const microWorker = require('./microWorker');
class RunWorker {
    constructor(details, store) {
        this.details = details;
        this.store = store;
        this.logger = logger_1.LoggerFactory.getLogger('RunServiceWorker');
        this.logger.trace(this.details, this.store);
    }
    async process(message) {
        try {
            const payload = message.payload;
            this.validate(payload);
            const type = this.parseTypeFromPayload(payload);
            const ids = this.parseIdsFromPayload(payload);
            const createRunCommand = new CreateRunCommand_1.CreateRunCommand(type, ids);
            this.logger.debug((0, node_util_1.inspect)(createRunCommand));
            const workerPromisify = (0, node_util_1.promisify)(microWorker).bind(microWorker);
            await workerPromisify(message, createRunCommand);
        }
        catch (error) {
            this.logger.error(`Failed to process message with err ${(0, node_util_1.inspect)(error)}`);
        }
    }
    validate(payload) {
        if (!(payload?.resourceVersionIds || payload?.pipelineStepId || payload?.pipelineStepIds)) {
            throw new errors_1.InvalidMessageError(`Message validation failed with payload ${(0, node_util_1.inspect)(payload)}`);
        }
    }
    parseTypeFromPayload(payload) {
        if (payload.pipelineStepId) {
            return 'pipelineStepId';
        }
        if (payload.pipelineStepIds) {
            return 'pipelineStepIds';
        }
        if (payload.resourceVersionIds) {
            return 'resourceVersionIds';
        }
        throw new errors_1.InvalidMessageError('No valid object type found in payload');
    }
    parseIdsFromPayload(payload) {
        if (payload.pipelineStepId) {
            return [payload.pipelineStepId];
        }
        if (payload.pipelineStepIds) {
            return payload.pipelineStepIds;
        }
        if (payload.resourceVersionIds) {
            return payload.resourceVersionIds;
        }
        throw new errors_1.InvalidMessageError('No valid object ids found in payload');
    }
}
exports.RunWorker = RunWorker;
//# sourceMappingURL=RunWorker.js.map