"use strict";
const Adapter = require('../../_global/pipelines/Adapter.js');
const runtrigger = require('../microWorker');
const _ = require('underscore');
const util = require('underscore');
describe('RunTrigger tests', () => {
    beforeAll(() => {
        setGlobal();
    });
    it('Test getResources for large amount of resource versions ids', async (done) => {
        let getResources = jest.fn().mockImplementation((query, callback) => {
            return callback(null, []);
        });
        let getConnections = jest.fn().mockImplementation((query, callback) => {
            return callback(null, []);
        });
        let totalResources = 1000;
        let getResourceVersions = jest.fn().mockImplementation((query, callback) => {
            const resourceVersionsMock = [];
            for (let i = 0; i < totalResources; i++) {
                resourceVersionsMock.push({ projectId: 1, resourceId: i });
            }
            callback(null, resourceVersionsMock);
        });
        Adapter.prototype.getResources = getResources;
        Adapter.prototype.getResourceVersions = getResourceVersions;
        Adapter.prototype.getPipelineStepConnections = getConnections;
        runtrigger({ payload: { resourceVersionIds: [1] } }, () => { });
        expect(getResources).toHaveBeenCalledTimes(4);
        const callsMap = new Map();
        let bucketIds = -1;
        for (let i = 0; i < totalResources; i++) {
            if (i % global.config.defaultBatchSize === 0) {
                bucketIds++;
                callsMap.set(bucketIds, []);
            }
            callsMap.get(bucketIds).push(i);
        }
        callsMap.forEach((value, key) => {
            expect(getResources.mock.calls[key][0]).toMatch(global.util.format('resourceIds=%s', value.join(',')));
        });
        return done();
    });
});
function getLogger() {
    return {
        error: jest.fn(),
        debug: jest.fn(),
        trace: jest.fn(),
        warn: jest.fn(),
        info: jest.fn()
    };
}
function setGlobal() {
    global.logger = getLogger();
    global.config = {};
    global.config.defaultBatchSize = 300;
    global.util = require('util');
    global.async = require('async');
    global._ = require('underscore');
    global.msName = 'runTrigger';
}
//# sourceMappingURL=runtrigger.test.js.map