'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var self = createRun;
module.exports = self;
var ApiAdapter = require('../Adapter.js');
var StepConsoleAdapter = require('../stepConsoleAdapter.js');
var BatchStepletConsoleAdapter = require('../BatchStepletConsoleAdapter.js');
const { XRAY_ENABLED_STEPS, XRAY_SCAN_STEP, HELLO_WORLD_TEMPLATE_NAME, JFROG_TEMPLATE_NAMESPACE, PRE_RUN_HOOK_STEP_INPUT_STEP_STATUSES, MATRIX_STEP } = require('../constants.js');
const { supportedOsAndFamilyMap } = require('../nodeMetadataMap');
var normalizeStep = require('./normalizeStep.js');
const createHeadJson_1 = require("./pipeInfoHandlers/createHeadJson");
const createStepJson_1 = require("./pipeInfoHandlers/createStepJson");
const createInfoJson_1 = require("./pipeInfoHandlers/createInfoJson");
var { CALL_OBJECT_LIMIT, CALL_OBJECT_LIMIT_FOR_BATCH } = require('../constants.js');
const { ConfigFactory } = require('@jfrog-pipelines/configuration');
const { inspect } = require('util');
const { RunAdjacencyComputerChain } = require('../runAdjacency/RunAdjacencyComputerChain');
const { writeFileSync } = require('fs');
const { AssemblerFactory } = require('../runAdjacency/AssemblerFactory.js');
const { DataLoaderFactory } = require('../runAdjacency/dataLoader/DataLoaderFactory.js');
const { RunAdjacencyDao } = require('../RunRepository.js');
var _ = require('underscore');
const util = require('util');
const async = require('async');
var finalStatesMap = {
    'success': 'Successful',
    'failure': 'Failed',
    'error': 'Error',
    'skipped': 'Skipped',
    'timeout': 'TimedOut',
    'cancelled': 'Cancelled',
    'unstable': 'Unstable'
};
var externalStepTypes = ['Jenkins'];
function createRun(jFrogPipelinesSessionId, inPayload, pipelineStepList, pipelineId, triggersByPipelineStepId, callback) {
    var bag = {
        inPayload: inPayload,
        startingPipelineSteps: pipelineStepList,
        pipelineId: pipelineId,
        connections: [],
        runResourceVersions: [],
        operationResources: [],
        steps: [],
        triggeredByResourceId: null,
        triggersByPipelineStepId: _.clone(triggersByPipelineStepId),
        jFrogPipelinesSessionId: jFrogPipelinesSessionId
    };
    bag.who = util.format('runTrigger|%s|jFrogPipelinesSessionId:%s|' +
        'pipelineId:%s', self.name, bag.jFrogPipelinesSessionId, bag.pipelineId);
    logger.debug('Inside', bag.who);
    async.series([
        _checkInputParams.bind(null, bag),
        _getPipelineById.bind(null, bag),
        _getProjectById.bind(null, bag),
        _getPipelineSourceById.bind(null, bag),
        _getNodePools.bind(null, bag),
        _getInternalIntegrations.bind(null, bag),
        _getProjectIntegrations.bind(null, bag),
        _getMasterIntegrations.bind(null, bag),
        _getAllConnections.bind(null, bag),
        _getPipelineSteps.bind(null, bag),
        _getDownstreamStepIds.bind(null, bag),
        _getRunHookPipelineSteps.bind(null, bag),
        _createRunHookObjects.bind(null, bag),
        _getInputSteps.bind(null, bag),
        _getInputStepsMasterResources.bind(null, bag),
        _getTriggeredByIdentity.bind(null, bag),
        _getEnvironmentVariables.bind(null, bag),
        _createRun.bind(null, bag),
        _getRunArtifactUrls.bind(null, bag),
        _createRunObjects.bind(null, bag),
        _createRunStepConnections.bind(null, bag),
        _createRunAdjacencyRecords.bind(null, bag),
        _createHeadJson.bind(null, bag),
        _queueReqSealerMessage.bind(null, bag),
        _createInfoJson.bind(null, bag),
        _updateRun.bind(null, bag),
        _updatePipelinesLatestStatusRunId.bind(null, bag),
        _skipStepsInCaseOfCriticalError.bind(null, bag),
        _queueStepTriggerMessage.bind(null, bag)
    ], function (err) {
        return callback(err, bag);
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.trace(who, 'Inside');
    bag.suAdapter = new ApiAdapter(bag.jFrogPipelinesSessionId, global.config.apiToken);
    return next();
}
function _getPipelineById(bag, next) {
    var who = bag.who + '|' + _getPipelineById.name;
    logger.trace(who, 'Inside');
    bag.suAdapter.getPipelineById(bag.pipelineId, function (err, pipeline) {
        if (err) {
            logger.warn(who, util.format('Failed to getPipelineById for pipelineId: %s with error: %s', bag.pipelineId, err));
            return next(true);
        }
        bag.pipeline = pipeline;
        return next();
    });
}
function _getProjectById(bag, next) {
    var who = bag.who + '|' + _getProjectById.name;
    logger.trace(who, 'Inside');
    bag.suAdapter.getProjectById(bag.pipeline.projectId, function (err, project) {
        if (err) {
            logger.warn(who, util.format('Failed to getProjectById for projectId: %s with error: %s', bag.pipeline.projectId, err));
            return next(true);
        }
        bag.project = project;
        bag.defaultNodePoolId = bag.project.configPropertyBag &&
            bag.project.configPropertyBag.defaultNodePoolId;
        return next();
    });
}
function _getPipelineSourceById(bag, next) {
    var who = bag.who + '|' + _getPipelineSourceById.name;
    logger.trace(who, 'Inside');
    bag.suAdapter.getPipelineSourceById(bag.pipeline.pipelineSourceId, function (err, pipelineSource) {
        if (err) {
            logger.warn(who, util.format('Failed to getPipelineSourceById for id: %s with error: %s', bag.pipeline.pipelineSourceId, err));
            return next(true);
        }
        bag.pipelineSource = pipelineSource;
        return next();
    });
}
function _getNodePools(bag, next) {
    var who = bag.who + '|' + _getNodePools.name;
    logger.trace(who, 'Inside');
    var query = util.format('projectKeys=%s', bag.project.sourceId);
    if (bag.project.name === 'default')
        query = util.format('%s&pipelineSourceIds=%s', query, bag.pipeline.pipelineSourceId);
    bag.suAdapter.getNodePools(query, function (err, nodePools) {
        if (err) {
            logger.warn(who, util.format('Failed to getNodePools for query: %s with error: %s', query, err));
            return next(true);
        }
        if (bag.project.name !== 'default') {
            nodePools = _.filter(nodePools, function (nodePool) {
                return (nodePool.environments == null) ||
                    (nodePool.environments && bag.pipelineSource.environments &&
                        nodePool.environments.some(v => bag.pipelineSource.environments.indexOf(v) !== -1));
            });
        }
        bag.nodePools = nodePools;
        return next();
    });
}
function _getInternalIntegrations(bag, next) {
    var who = bag.who + '|' + _getInternalIntegrations.name;
    logger.trace(who, 'Inside');
    var query = util.format('isInternal=true');
    var fetchedAllIntegrations = false;
    var skip = 0;
    var internalIntegrations = [];
    async.whilst(function () {
        return !fetchedAllIntegrations;
    }, function (nextPage) {
        var currentQuery = util.format('%s&sortBy=id&sortOrder=1&limit=%s&skip=%s', query, global.config.defaultPageSize, skip);
        bag.suAdapter.getProjectIntegrations(currentQuery, function (err, integrations) {
            if (err) {
                logger.warn(who, util.format('Failed to getProjectIntegrations for query: %s with error: %s', query, err));
                return nextPage(true);
            }
            if (integrations.length === global.config.defaultPageSize)
                skip = skip + global.config.defaultPageSize;
            else
                fetchedAllIntegrations = true;
            internalIntegrations = internalIntegrations.concat(integrations);
            return nextPage();
        });
    }, function (err) {
        if (err)
            return next(err);
        bag.internalIntegrationsById = _.indexBy(internalIntegrations, 'id');
        return next();
    });
}
function _getProjectIntegrations(bag, next) {
    var who = bag.who + '|' + _getProjectIntegrations.name;
    logger.trace(who, 'Inside');
    var query = util.format('projectKeys=%s', bag.project.sourceId);
    if (bag.project.name === 'default')
        query = util.format('%s&pipelineSourceIds=%s', query, bag.pipeline.pipelineSourceId);
    var fetchedAllIntegrations = false;
    var skip = 0;
    var projectIntegrations = [];
    async.whilst(function () {
        return !fetchedAllIntegrations;
    }, function (nextPage) {
        var currentQuery = util.format('%s&sortBy=id&sortOrder=1&limit=%s&skip=%s', query, global.config.defaultPageSize, skip);
        bag.suAdapter.getProjectIntegrations(currentQuery, function (err, integrations) {
            if (err) {
                logger.warn(who, util.format('Failed to getProjectIntegrations for query: %s with error: %s', query, err));
                return nextPage(true);
            }
            if (integrations.length === global.config.defaultPageSize)
                skip = skip + global.config.defaultPageSize;
            else
                fetchedAllIntegrations = true;
            projectIntegrations = projectIntegrations.concat(integrations);
            return nextPage();
        });
    }, function (err) {
        if (err)
            return next(err);
        if (bag.project.name !== 'default') {
            projectIntegrations = _.filter(projectIntegrations, function (integration) {
                return (integration.environments == null) ||
                    (integration.environments && bag.pipelineSource.environments &&
                        integration.environments.some(v => bag.pipelineSource.environments.indexOf(v) !== -1));
            });
        }
        bag.masterIntegrationIds =
            _.uniq(_.compact(_.pluck(projectIntegrations, 'masterIntegrationId')));
        bag.projectIntegrationsById = _.indexBy(projectIntegrations, 'id');
        bag.projectIntegrationsByName = _.indexBy(projectIntegrations, 'name');
        return next();
    });
}
function _getMasterIntegrations(bag, next) {
    var who = bag.who + '|' + _getMasterIntegrations.name;
    logger.trace(who, 'Inside');
    var query = 'masterIntegrationIds=' + bag.masterIntegrationIds.join(',');
    bag.suAdapter.getMasterIntegrations(query, function (err, masterIntegrations) {
        if (err) {
            logger.warn(who, util.format('Failed to getMasterIntegrations for query: %s with error: %s', query, err));
            return next(err);
        }
        bag.masterIntegrationsById = _.indexBy(masterIntegrations, 'id');
        return next();
    });
}
function _getAllConnections(bag, next) {
    var who = bag.who + '|' + _getAllConnections.name;
    logger.debug(who, 'Inside');
    var query = util.format('projectIds=%s&pipelineIds=%s&sortOrder=1&sortBy=id', bag.pipeline.projectId, bag.pipeline.id);
    var fetchedAllConnections = false;
    var skip = 0;
    async.whilst(function () {
        return !fetchedAllConnections;
    }, function (done) {
        var currentQuery = util.format('%s&limit=%s&skip=%s', query, global.config.defaultPageSize, skip);
        bag.suAdapter.getPipelineStepConnections(currentQuery, function (err, connections) {
            if (err) {
                logger.warn(who, util.format('Failed to ' +
                    'getPipelineStepConnections for query: %s with err: %s', query, err));
                return done(true);
            }
            if (connections.length === global.config.defaultPageSize) {
                skip = skip + global.config.defaultPageSize;
            }
            else {
                fetchedAllConnections = true;
            }
            bag.connections = bag.connections.concat(connections);
            return done();
        });
    }, function (err) {
        return next(err);
    });
}
function _getPipelineSteps(bag, next) {
    var who = bag.who + '|' + _getPipelineSteps.name;
    logger.trace(who, 'Inside');
    var startingPipelineStepIds = _.pluck(bag.startingPipelineSteps, 'id');
    var pipelineStepIds = _.pluck(bag.connections, 'pipelineStepId');
    pipelineStepIds = _.union(pipelineStepIds, startingPipelineStepIds);
    var stepIdBatches = [];
    var i = 0;
    while (i < pipelineStepIds.length) {
        stepIdBatches.push(pipelineStepIds.slice(i, i + global.config.defaultBatchSize));
        i += global.config.defaultBatchSize;
    }
    var pipelineSteps = [];
    async.eachLimit(stepIdBatches, 10, function (stepIdBatch, nextBatch) {
        var query = util.format('projectIds=%s&pipelineStepIds=%s&limit=%s', bag.project.id, stepIdBatch.join(','), global.config.defaultBatchSize);
        bag.suAdapter.getPipelineSteps(query, function (err, response) {
            if (err) {
                logger.warn(who, util.format('Failed to getPipelineSteps for query: %s with error: %s', query, err));
                return nextBatch(true);
            }
            pipelineSteps = pipelineSteps.concat(response);
            return process.nextTick(nextBatch);
        });
    }, function (err) {
        if (err)
            return next(err);
        bag.pipelineStepsById = _.indexBy(pipelineSteps, 'id');
        bag.pipelineSteps = pipelineSteps;
        return next();
    });
}
function _getDownstreamStepIds(bag, next) {
    var who = bag.who + '|' + _getDownstreamStepIds.name;
    logger.debug(who, 'Inside');
    bag.runPipelineStepIds = [];
    var resourceTriggers = {};
    bag.startingPipelineSteps = _.reject(bag.startingPipelineSteps, function (pipelineStep) {
        return _.includes([global.systemCodesByName.PreRun.code,
            global.systemCodesByName.PostRun.code], pipelineStep.typeCode);
    });
    var queue = _.map(bag.startingPipelineSteps, function (item) {
        return {
            pipelineStepId: item.id
        };
    });
    const firstSteps = new Set(_.pluck(queue, 'pipelineStepId'));
    const lastSteps = new Set();
    while (!_.isEmpty(queue)) {
        var element = queue.shift();
        if (element.pipelineStepId) {
            var pStep = bag.pipelineStepsById[element.pipelineStepId];
            if (pStep && !pStep.isDeleted && pStep.isConsistent) {
                bag.runPipelineStepIds.push(element.pipelineStepId);
                var stepDownstreamINConnections = _.where(bag.connections, {
                    isTrigger: true,
                    operation: 'IN',
                    operationPipelineStepId: element.pipelineStepId
                });
                if (_.isEmpty(stepDownstreamINConnections)) {
                    lastSteps.add(element.pipelineStepId);
                }
                _.each(stepDownstreamINConnections, function (connection) {
                    firstSteps.delete(connection.pipelineStepId);
                    if (_.contains(bag.runPipelineStepIds, connection.pipelineStepId))
                        return;
                    bag.triggersByPipelineStepId[connection.pipelineStepId] =
                        bag.triggersByPipelineStepId[element.pipelineStepId];
                    queue.push({
                        pipelineStepId: connection.pipelineStepId
                    });
                });
                var downstreamOUTConnections = _.where(bag.connections, {
                    operation: 'OUT',
                    pipelineStepId: element.pipelineStepId
                });
                _.each(downstreamOUTConnections, function (connection) {
                    resourceTriggers[connection.operationResourceId] =
                        bag.triggersByPipelineStepId[element.pipelineStepId];
                    queue.push({
                        resourceId: connection.operationResourceId,
                        connectedPipelineStepId: element.pipelineStepId
                    });
                });
            }
        }
        else if (element.resourceId) {
            var resDownstreamINConnections = _.where(bag.connections, {
                isTrigger: true,
                operation: 'IN',
                operationResourceId: element.resourceId
            });
            if (!_.isEmpty(resDownstreamINConnections)) {
                lastSteps.delete(element.connectedPipelineStepId);
            }
            _.each(resDownstreamINConnections, function (connection) {
                firstSteps.delete(connection.pipelineStepId);
                if (_.contains(bag.runPipelineStepIds, connection.pipelineStepId))
                    return;
                bag.triggersByPipelineStepId[connection.pipelineStepId] =
                    resourceTriggers[element.resourceId];
                queue.push({
                    pipelineStepId: connection.pipelineStepId
                });
            });
        }
    }
    bag.runPipelineStepIds = _.uniq(bag.runPipelineStepIds);
    bag.firstStepIds = [...firstSteps.values()];
    bag.lastStepIds = [...lastSteps.values()];
    return next();
}
function _getRunHookPipelineSteps(bag, next) {
    var who = bag.who + '|' + _getRunHookPipelineSteps.name;
    logger.debug(who, 'Inside');
    const preRunTypeCode = global.systemCodesByName.PreRun.code;
    const postRunTypeCode = global.systemCodesByName.PostRun.code;
    const runHooksStepTypes = [preRunTypeCode, postRunTypeCode];
    const query = util.format(`typeCodes=${runHooksStepTypes.join(',')}&pipelineIds=${bag.pipeline.id}`);
    bag.suAdapter.getPipelineSteps(query, function (err, runHookPipelineSteps) {
        if (err) {
            logger.warn(who, util.format('Failed to getPipelineSteps for query: %s with error: %s', query, err));
            return next(true);
        }
        bag.pipelineSteps = bag.pipelineSteps.concat(runHookPipelineSteps);
        bag.pipelineStepsById = _.indexBy(bag.pipelineSteps, 'id');
        const runHookPipelineStepsByCode = _.indexBy(runHookPipelineSteps || [], 'typeCode');
        bag.preRunHookStep = runHookPipelineStepsByCode[preRunTypeCode];
        bag.postRunHookStep = runHookPipelineStepsByCode[postRunTypeCode];
        return next();
    });
}
function _createRunHookObjects(bag, next) {
    if (!bag.preRunHookStep && !bag.postRunHookStep)
        return next();
    var who = bag.who + '|' + _createRunHookObjects.name;
    logger.warn(who, 'Inside');
    if (bag.preRunHookStep) {
        createPreRunHookConnections(bag, bag.preRunHookStep);
        bag.runPipelineStepIds = [bag.preRunHookStep.id, ...bag.runPipelineStepIds];
    }
    if (bag.postRunHookStep) {
        createPostRunHookConnections(bag, bag.postRunHookStep);
        bag.runPipelineStepIds = [...bag.runPipelineStepIds, bag.postRunHookStep.id];
    }
    return next();
}
function createPreRunHookConnections(bag, preRunHookStep) {
    _.each(bag.firstStepIds, function (firstStepId) {
        const pipelineStep = bag.pipelineStepsById[firstStepId];
        if (!pipelineStep) {
            return;
        }
        const inputSteps = [
            {
                name: preRunHookStep.name,
                status: PRE_RUN_HOOK_STEP_INPUT_STEP_STATUSES
            }
        ];
        if (pipelineStep?.ymlConfigPropertyBag?.inputSteps) {
            pipelineStep.ymlConfigPropertyBag = {
                inputSteps: [...inputSteps, ...pipelineStep?.ymlConfigPropertyBag?.inputSteps]
            };
        }
        else if (pipelineStep?.ymlConfigPropertyBag) {
            pipelineStep.ymlConfigPropertyBag.inputSteps = inputSteps;
        }
        else if (!pipelineStep.ymlConfigPropertyBag) {
            pipelineStep.ymlConfigPropertyBag = {
                inputSteps: inputSteps
            };
        }
        bag.connections.push({
            pipelineStepId: pipelineStep.id,
            operation: "IN",
            operationPipelineStepId: preRunHookStep.id,
            operationPipelineStepName: preRunHookStep.name,
            projectId: bag.pipeline.projectId,
            pipelineSourceId: bag.pipeline.pipelineSourceId,
            pipelineId: bag.pipeline.id,
            isPipelineConnection: false,
            isTrigger: true,
        });
    });
}
function createPostRunHookConnections(bag, postRunStep) {
    const inputSteps = [];
    _.each(bag.lastStepIds, function (lastStepId) {
        const pipelineStep = bag.pipelineStepsById[lastStepId];
        if (!pipelineStep) {
            return;
        }
        inputSteps.push({
            name: pipelineStep.name,
            status: PRE_RUN_HOOK_STEP_INPUT_STEP_STATUSES
        });
        bag.connections.push({
            pipelineStepId: postRunStep.id,
            operation: "IN",
            operationPipelineStepId: pipelineStep.id,
            operationPipelineStepName: pipelineStep.name,
            projectId: bag.pipeline.projectId,
            pipelineSourceId: bag.pipeline.pipelineSourceId,
            pipelineId: bag.pipeline.id,
            isPipelineConnection: false,
            isTrigger: true,
        });
    });
    if (postRunStep?.ymlConfigPropertyBag) {
        postRunStep.ymlConfigPropertyBag.inputSteps = inputSteps;
    }
    else {
        postRunStep.ymlConfigPropertyBag = {
            inputSteps: inputSteps
        };
    }
}
function _getInputSteps(bag, next) {
    var who = bag.who + '|' + _getInputSteps.name;
    logger.trace(who, 'Inside');
    var inputStepsToFetch = [], inputStepsToFetchMap = {};
    bag.pipelineStepsByName = {};
    var connectionsByPipelineStepId = _.groupBy(bag.connections, 'pipelineStepId');
    _.each(bag.runPipelineStepIds, function (runPipelineStepId) {
        _.each(connectionsByPipelineStepId[runPipelineStepId], function (connection) {
            if (connection.operation !== 'IN')
                return;
            if (!connection.operationPipelineStepId)
                return;
            var inputStepId = connection.operationPipelineStepId;
            var inputStepName = connection.operationPipelineStepName;
            if (inputStepsToFetchMap[inputStepId])
                return;
            if (bag.pipelineStepsByName[inputStepName])
                return;
            if (bag.pipelineStepsById[inputStepId])
                bag.pipelineStepsByName[inputStepName] =
                    bag.pipelineStepsById[inputStepId];
            else
                inputStepsToFetchMap[inputStepId] = {};
        });
    });
    inputStepsToFetch = Object.keys(inputStepsToFetchMap);
    if (_.isEmpty(inputStepsToFetch))
        return next();
    var inputStepsToFetchBatches = [], inputStepsFetched = [];
    var i = 0;
    while (i < inputStepsToFetch.length) {
        inputStepsToFetchBatches.push(inputStepsToFetch.slice(i, i + global.config.defaultBatchSize));
        i += global.config.defaultBatchSize;
    }
    async.eachLimit(inputStepsToFetchBatches, 10, function (inputStepsToFetchBatch, nextBatch) {
        var query = util.format('projectIds=%s&pipelineStepIds=%s&limit=%s', bag.project.id, inputStepsToFetchBatch.join(','), global.config.defaultBatchSize);
        bag.suAdapter.getPipelineSteps(query, function (err, pipelineSteps) {
            if (err) {
                logger.warn(who, util.format('Failed to getPipelineSteps for query: %s with error: %s', query, err));
                return next(true);
            }
            if (inputStepsToFetchBatch.length !== pipelineSteps.length) {
                var missingPipelineSteps = _.difference(inputStepsToFetchBatch, _.pluck(pipelineSteps, 'id'));
                logger.warn(who, util.format('Expected %s pipelineSteps, got %s pipelineSteps. Missing ' +
                    'pipelineSteps: %s', inputStepsToFetchBatch.length, pipelineSteps.length, missingPipelineSteps.join(', ')));
                return next(true);
            }
            inputStepsFetched = inputStepsFetched.concat(pipelineSteps);
            return process.nextTick(nextBatch);
        });
    }, function (err) {
        if (!err)
            _.extend(bag.pipelineStepsByName, _.indexBy(inputStepsFetched, 'name'));
        return next(err);
    });
}
function _getInputStepsMasterResources(bag, next) {
    if (!global.config.signedPipelinesEnabled)
        return next();
    var who = bag.who + '|' + _getInputStepsMasterResources.name;
    logger.trace(who, 'Inside');
    bag.masterResourcesById = {};
    var masterResourceIds = [];
    masterResourceIds = _.uniq(_.compact(_.pluck(_.values(bag.pipelineStepsByName), 'masterResourceId')));
    if (_.isEmpty(masterResourceIds))
        return next();
    var masterResourceIdBatches = [];
    var i = 0;
    while (i < masterResourceIds.length) {
        masterResourceIdBatches.push(masterResourceIds.slice(i, i + global.config.defaultBatchSize));
        i += global.config.defaultBatchSize;
    }
    var masterResources = [];
    async.eachLimit(masterResourceIdBatches, 10, function (masterResourceIdBatch, nextBatch) {
        var query = util.format('masterResourceIds=%s&limit=%s', masterResourceIdBatch.join(','), global.config.defaultBatchSize);
        bag.suAdapter.getMasterResources(query, function (err, response) {
            if (err) {
                logger.warn(who, util.format('Failed to getMasterResources for query: %s with error: %s', query, err));
                return nextBatch(true);
            }
            masterResources = masterResources.concat(response);
            return process.nextTick(nextBatch);
        });
    }, function (err) {
        if (err)
            return next(err);
        bag.masterResourcesById = _.indexBy(masterResources, 'id');
        return next();
    });
}
function _getTriggeredByIdentity(bag, next) {
    if (!bag.inPayload.pipelineStepId && !bag.inPayload.pipelineStepIds)
        return next();
    var who = bag.who + '|' + _getTriggeredByIdentity.name;
    logger.trace(who, 'Inside');
    bag.suAdapter.getIdentityById(bag.inPayload.callerId, function (err, identity) {
        if (err) {
            logger.warn(who, util.format('Failed to getIdentityById for identityId: %s with error: %s', bag.inPayload.callerId, err));
            return next(true);
        }
        bag.triggeredByIdentity = identity;
        return next();
    });
}
function _getEnvironmentVariables(bag, next) {
    var who = bag.who + '|' + _getEnvironmentVariables.name;
    logger.trace(who, 'Inside');
    var fetchedAllEnvironmentVariables = false;
    var skip = 0;
    var environmentVariables = [];
    let query = '';
    async.whilst(function () {
        return !fetchedAllEnvironmentVariables;
    }, function (nextPage) {
        var currentQuery = util.format('sortBy=id&sortOrder=1&limit=%s&skip=%s', global.config.defaultPageSize, skip);
        bag.suAdapter.getEnvironmentVariables(currentQuery, function (err, variables) {
            if (err) {
                logger.warn(who, util.format('Failed to getEnvironmentVariables for query: %s with error: %s', query, err));
                return nextPage(true);
            }
            if (variables.length === global.config.defaultPageSize)
                skip = skip + global.config.defaultPageSize;
            else
                fetchedAllEnvironmentVariables = true;
            environmentVariables = environmentVariables.concat(variables);
            return nextPage();
        });
    }, function (err) {
        if (err)
            return next(err);
        const environmentVariablesByScope = _.groupBy(environmentVariables, 'scope');
        bag.globalEnvironmentVariables = environmentVariablesByScope['global'] || [];
        bag.projectEnvironmentVariablesByProjectId = _.groupBy(environmentVariablesByScope['project'] || [], 'projectId');
        return next();
    });
}
function _createRun(bag, next) {
    var who = bag.who + '|' + _createRun.name;
    logger.debug(who, 'Inside');
    var newRun = {
        projectId: bag.project.id,
        pipelineSourceId: bag.pipeline.pipelineSourceId,
        pipelineId: bag.pipeline.id,
        statusCode: global.systemCodesByName.creating.code,
        staticPropertyBag: {},
        triggerId: bag.inPayload.triggerId,
    };
    if (bag.inPayload.configPropertyBag &&
        bag.inPayload.configPropertyBag.buildUser)
        newRun.staticPropertyBag = {
            triggeredByUserName: bag.inPayload.configPropertyBag.buildUser
        };
    else if (bag.triggeredByIdentity)
        newRun.staticPropertyBag = {
            triggeredByUserName: bag.triggeredByIdentity.name
        };
    if (global.config.signedPipelinesEnabled)
        newRun.staticPropertyBag.signedPipelinesEnabled = true;
    if (global.config.autoSyncResourceIfOutdated)
        newRun.staticPropertyBag.autoSyncResourceIfOutdated = true;
    var triggeredByResourceName;
    var triggeredByRunId;
    _.some(bag.startingPipelineSteps, function (startingStep) {
        var trigger = bag.triggersByPipelineStepId[startingStep.id];
        if (trigger) {
            bag.triggeredByResourceId = trigger.resourceId;
            triggeredByRunId = trigger.parentRunId;
            triggeredByResourceName = trigger.resourceName;
            return true;
        }
    });
    if (triggeredByResourceName) {
        newRun.staticPropertyBag = {
            triggeredByResourceName: triggeredByResourceName
        };
        newRun.parentRunId = triggeredByRunId;
        bag.triggeredByResourceName = triggeredByResourceName;
    }
    bag.suAdapter.postRun(newRun, function (err, run) {
        if (err) {
            logger.warn(who, util.format('Failed to postRun for pipelineId: %s with error: %s', bag.pipeline.id, err));
            return next(true);
        }
        bag.run = run;
        return next();
    });
}
function _getRunArtifactUrls(bag, next) {
    var who = bag.who + '|' + _getRunArtifactUrls.name;
    logger.debug(who, 'Inside');
    bag.suAdapter.getArtifactUrlByRunId(bag.run.id, '', function (err) {
        if (err) {
            logger.warn(who, util.format('Failed to getArtifactUrl for runId: %s with error: %s', bag.run.id, err));
            bag.stateUnavailable = true;
        }
        return next();
    });
}
function _createRunObjects(bag, next) {
    var who = bag.who + '|' + _createRunObjects.name;
    logger.trace(who, 'Inside');
    bag.isRunReady = false;
    bag.isCriticalErr = false;
    const externalStepTypesSet = new Set(externalStepTypes);
    const injectedNodePools = bag.inPayload.injectedNodePools || {};
    const pipelineLevelNodePool = injectedNodePools.nodePool;
    const injectNodePoolsByStepName = _.indexBy(injectedNodePools.steps || [], 'stepName');
    bag.steplets = [];
    async.eachSeries(bag.runPipelineStepIds, function (pipelineStepId, nextPipelineStepId) {
        var seriesBag = {
            who: bag.who + '|pipelineStepId:' + pipelineStepId,
            suAdapter: bag.suAdapter,
            inPayload: bag.inPayload,
            firstPipelineStepId: bag.inPayload.pipelineStepId,
            pipelineStepId: pipelineStepId,
            pipelineStep: bag.pipelineStepsById[pipelineStepId],
            runId: bag.run.id,
            runNumber: bag.run.runNumber,
            pipeline: bag.pipeline,
            pipelineSource: bag.pipelineSource,
            nodePools: bag.nodePools,
            internalIntegrationsById: bag.internalIntegrationsById,
            projectIntegrationsById: bag.projectIntegrationsById,
            projectIntegrationsByName: bag.projectIntegrationsByName,
            masterIntegrationsById: bag.masterIntegrationsById,
            consoleErrors: [],
            defaultNodePoolId: bag.defaultNodePoolId,
            masterResourcesById: bag.masterResourcesById,
            pipelineStepsByName: bag.pipelineStepsByName,
            runResourceVersions: [],
            triggeredByIdentity: bag.triggeredByIdentity,
            triggersByPipelineStepId: bag.triggersByPipelineStepId,
            globalEnvironmentVariables: bag.globalEnvironmentVariables || [],
            projectEnvironmentVariables: (bag.projectEnvironmentVariablesByProjectId ?
                (bag.projectEnvironmentVariablesByProjectId[bag.project.id] || []) : []),
            jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId,
        };
        let injectedNodePool = null;
        if (pipelineLevelNodePool) {
            injectedNodePool = pipelineLevelNodePool;
        }
        const pipelineStep = bag.pipelineStepsById[pipelineStepId] || {};
        if (pipelineStep &&
            injectNodePoolsByStepName[pipelineStep.name] &&
            injectNodePoolsByStepName[pipelineStep.name].nodePool) {
            injectedNodePool = injectNodePoolsByStepName[pipelineStep.name].nodePool;
        }
        seriesBag.injectNodePool = injectedNodePool;
        if (bag.stateUnavailable) {
            if (bag.project.name === 'default')
                seriesBag.consoleErrors.push('Artifactory repository with name ' +
                    global.config.rootBucket + ' unavailable.');
            else
                seriesBag.consoleErrors.push('Artifactory repository with name ' +
                    global.config.rootBucket + ' in project ' +
                    bag.project.name + ' unavailable.');
        }
        seriesBag.connections = _.where(bag.connections, {
            pipelineStepId: pipelineStepId
        });
        if (seriesBag.pipelineStep)
            seriesBag.isExternalStep = externalStepTypesSet.has(global.systemCodesByCode[seriesBag.pipelineStep.typeCode].name);
        if (bag.inPayload.configPropertyBag &&
            pipelineStepId === bag.inPayload.pipelineStepId)
            seriesBag.payloadConfigPropertyBag = bag.inPayload.configPropertyBag;
        if (bag.inPayload.externalBuildId &&
            pipelineStepId === bag.inPayload.pipelineStepId)
            seriesBag.externalBuildId = bag.inPayload.externalBuildId;
        if (bag.inPayload.externalBuildUrl &&
            pipelineStepId === bag.inPayload.pipelineStepId)
            seriesBag.externalBuildUrl = bag.inPayload.externalBuildUrl;
        async.series([
            __getOperationResources.bind(null, seriesBag),
            __getMasterResources.bind(null, seriesBag),
            __normalizeStep.bind(null, seriesBag),
            __checkNodePoolCompatibility.bind(null, seriesBag),
            __checkBetaNodePoolCompatibility.bind(null, seriesBag),
            __checkStepIntegrations.bind(null, seriesBag),
            __getOperationResourceVersions.bind(null, seriesBag),
            __getHelloWorldTemplate.bind(null, seriesBag),
            __createStep.bind(null, seriesBag),
            __createSteplets.bind(null, seriesBag),
            __createRunResourceVersions.bind(null, seriesBag),
            __errorOutStepIfResourceVersionPublishFailed.bind(null, seriesBag),
            __createStepJson.bind(null, seriesBag),
            __postStepConsoles.bind(null, seriesBag)
        ], function () {
            bag.runResourceVersions =
                bag.runResourceVersions.concat(seriesBag.runResourceVersions);
            bag.operationResources =
                bag.operationResources.concat(seriesBag.operationResources);
            bag.steps.push(seriesBag.step);
            bag.steplets = bag.steplets.concat(seriesBag.steplets);
            if (seriesBag.step) {
                bag.isRunReady = bag.isRunReady ||
                    (seriesBag.step.statusCode ===
                        global.systemCodesByName.waiting.code);
                bag.isCriticalErr = bag.isCriticalErr ||
                    (seriesBag.step.statusCode ===
                        global.systemCodesByName.error.code);
            }
            return nextPipelineStepId();
        });
    }, function () {
        return next();
    });
}
function _createRunStepConnections(bag, next) {
    var who = bag.who + '|' + _createRunStepConnections.name;
    logger.debug(who, 'Inside');
    var projectId = bag.project.id;
    var pipelineId = bag.pipeline.id;
    var pipelineSourceId = bag.pipeline.pipelineSourceId;
    var runId = bag.run.id;
    const stepByPipelineStepId = _.indexBy(bag.steps, 'pipelineStepId');
    const startingPipelineStepsById = _.indexBy(bag.startingPipelineSteps, 'id');
    const runPipelineStepsById = new Set(bag.runPipelineStepIds || []);
    bag.runStepConnections = [];
    async.eachLimit(bag.connections, 10, function (connection, nextConnection) {
        if (startingPipelineStepsById[connection.pipelineStepId] &&
            connection.operationPipelineStepId) {
            if (!runPipelineStepsById.has(connection.operationPipelineStepId))
                return process.nextTick(nextConnection);
        }
        const step = stepByPipelineStepId[connection.pipelineStepId];
        if (!step)
            return process.nextTick(nextConnection);
        var newRunConnection = {
            projectId: projectId,
            pipelineId: pipelineId,
            pipelineSourceId: pipelineSourceId,
            runId: runId,
            stepId: step.id,
            operation: connection.operation,
            isPipelineConnection: connection.isPipelineConnection || false,
            isTrigger: connection.isTrigger
        };
        if (connection.operationResourceId) {
            var runResourceVersion = _.findWhere(bag.runResourceVersions, {
                stepId: step.id,
                resourceId: connection.operationResourceId
            });
            if (!runResourceVersion)
                return process.nextTick(nextConnection);
            newRunConnection.operationRunResourceVersionId =
                runResourceVersion.id;
            newRunConnection.operationRunResourceVersionName =
                connection.operationResourceName;
        }
        else if (connection.operationPipelineStepId) {
            const pStep = stepByPipelineStepId[connection.operationPipelineStepId];
            if (!pStep)
                return process.nextTick(nextConnection);
            newRunConnection.operationStepId = pStep.id;
            newRunConnection.operationStepName =
                connection.operationPipelineStepName;
        }
        else if (connection.operationIntegrationId) {
            newRunConnection.operationIntegrationId =
                connection.operationIntegrationId;
            newRunConnection.operationIntegrationName =
                connection.operationIntegrationName;
        }
        bag.suAdapter.postRunStepConnection(newRunConnection, function (err, runStepConnection) {
            if (err) {
                logger.warn(who, util.format('Failed to postRunStepConnection for stepId: %s ' +
                    'with error: %s', newRunConnection.stepId, err));
                return nextConnection(true);
            }
            bag.runStepConnections.push(runStepConnection);
            return process.nextTick(nextConnection);
        });
    }, function (err) {
        return next(err);
    });
}
async function _createRunAdjacencyRecords(bag) {
    var who = bag.who + '|' + _createRunAdjacencyRecords.name;
    logger.debug(who, 'Inside');
    try {
        const [step, dependencyMode, chronological, dependencyNodes] = await AssemblerFactory.getAssemblers(bag.run, bag.pipeline, bag.steps, bag.steplets, bag.runResourceVersions, bag.runStepConnections, DataLoaderFactory.getDataLoaderWithPipelinesClient(global.config.apiToken));
        const runAdjacencyChain = new RunAdjacencyComputerChain(dependencyNodes, step, chronological, dependencyMode);
        const runAdjacencyRecords = runAdjacencyChain.compute();
        const runAdjacencyDao = new RunAdjacencyDao();
        const startRunTime = Date.now();
        await runAdjacencyDao.createBulk(runAdjacencyRecords);
        const stopRunTime = Date.now();
        const runTimeDiff = new Date(stopRunTime - startRunTime);
        logger.info(who, `Posted runAdjacencies records: ${runAdjacencyRecords.length} in ${runTimeDiff.valueOf()} ms`);
    }
    catch (error) {
        logger.error('Failed to post runAdajacencies with error :', util.inspect(error));
    }
}
function _createHeadJson(bag, next) {
    if (!global.config.signedPipelinesEnabled)
        return next();
    var who = bag.who + '|' + _createHeadJson.name;
    logger.debug(who, 'Inside');
    var payload = {
        run: bag.run,
        project: bag.project,
        pipelineSource: bag.pipelineSource,
        pipeline: bag.pipeline,
        projectIntegrationsById: bag.projectIntegrationsById,
        masterIntegrationsById: bag.masterIntegrationsById,
        operationResources: bag.operationResources,
        suAdapter: bag.suAdapter,
        jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId
    };
    const createHeadJson = new createHeadJson_1.CreateHeadJson(payload);
    createHeadJson.createPipeInfo()
        .catch(error => {
        logger.warn(util.format('createHeadJson failed for run: %s', bag.run.id));
    }).finally(() => {
        next();
    });
}
function _queueReqSealerMessage(bag, next) {
    if (!global.config.signedPipelinesEnabled)
        return next();
    var who = bag.who + '|' + _queueReqSealerMessage.name;
    logger.debug(who, 'Inside');
    var message = {
        where: 'core.reqSealer',
        payload: {
            jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId,
            objectType: 'run',
            objectId: bag.run.id,
            objectName: 'head'
        }
    };
    bag.suAdapter.postToVortex(message, function (err) {
        if (err)
            return next(new ActErr(who, err, 'postToVortex returned error'));
        return next();
    });
}
function _createInfoJson(bag, next) {
    if (!global.config.signedPipelinesEnabled)
        return next();
    var who = bag.who + '|' + _createInfoJson.name;
    logger.debug(who, 'Inside');
    const payload = {
        run: bag.run,
        pipeline: bag.pipeline,
        pipelineSource: bag.pipelineSource,
        jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId
    };
    const createInfoJson = new createInfoJson_1.CreateInfoJson(payload);
    createInfoJson.createPipeInfo()
        .catch(error => {
        logger.warn(util.format('createInfoJson failed for run: %s', bag.run.id));
    }).finally(() => {
        next();
    });
}
function _updateRun(bag, next) {
    var who = bag.who + '|' + _updateRun.name;
    logger.debug(who, 'Inside');
    var update = {};
    if (bag.isCriticalErr) {
        update.statusCode = global.systemCodesByName.error.code;
    }
    else if (bag.isRunReady) {
        update.statusCode = global.systemCodesByName.ready.code;
    }
    else {
        var steps = bag.steps;
        var stepStatusCodes = _.pluck(steps, 'statusCode');
        stepStatusCodes = _.sortBy(stepStatusCodes, function (statusCode) {
            var stepStatus = global.systemCodesByCode[statusCode].name;
            return global.statusCodes.statusPriorityMap[stepStatus];
        });
        update.statusCode = _.last(stepStatusCodes);
    }
    var staticPropertyBag = bag.run.staticPropertyBag;
    if (bag.triggeredByResourceId) {
        var triggeredByResource = _.findWhere(bag.runResourceVersions, { resourceId: bag.triggeredByResourceId });
        if (triggeredByResource) {
            staticPropertyBag.triggeredByRunResourceVersionId =
                triggeredByResource.id;
        }
        else {
            var msg = util.format('runResourceVersion with Id: %s is missing', bag.triggeredByResourceId);
            logger.warn(who, msg);
        }
        update.staticPropertyBag = staticPropertyBag;
    }
    bag.suAdapter.putRunById(bag.run.id, update, function (err, run) {
        if (err) {
            logger.warn(who, util.format('Failed to putRunById for id: %s with error: %s', bag.run.id, err));
            return next(true);
        }
        bag.run = run;
        return next();
    });
}
function _updatePipelinesLatestStatusRunId(bag, next) {
    if (_.isEmpty(finalStatesMap[global.systemCodesByCode[bag.run.statusCode].name]))
        return next();
    var who = bag.who + '|' + _updatePipelinesLatestStatusRunId.name;
    logger.debug(who, 'Inside');
    var update = {
        latestCompletedRunId: bag.run.id,
    };
    var status = finalStatesMap[global.systemCodesByCode[bag.run.statusCode].name];
    var latestStatusRunId = 'latest' + status + 'RunId';
    update[latestStatusRunId] = bag.run.id;
    bag.suAdapter.putPipelineById(bag.run.pipelineId, update, function (err) {
        if (err) {
            logger.warn(who, util.format('Failed to putPipelineById for id: %s with error: %s', bag.run.pipelineId, err));
            return next(true);
        }
        return next();
    });
}
function _skipStepsInCaseOfCriticalError(bag, next) {
    if (!bag.isCriticalErr)
        return next();
    let who = bag.who + '|' + _skipStepsInCaseOfCriticalError.name;
    logger.trace(who, 'Inside');
    async.eachLimit(bag.steps, 10, (step, nextStep) => {
        if (step.statusCode === global.systemCodesByName.error.code) {
            return process.nextTick(nextStep);
        }
        const update = {
            statusCode: global.systemCodesByName.skipped.code
        };
        bag.suAdapter.putStepById(step.id, update, function (err) {
            if (err) {
                logger.warn(who, util.format('Failed to putStepById for id: %s', step.id), err);
            }
            return process.nextTick(nextStep);
        });
    }, function () {
        return next();
    });
}
function _queueStepTriggerMessage(bag, next) {
    if (bag.isCriticalErr)
        return next();
    var who = bag.who + '|' + _queueStepTriggerMessage.name;
    logger.debug(who, 'Inside');
    var message = {
        where: 'core.stepTrigger',
        payload: {
            jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId
        }
    };
    bag.suAdapter.postToVortex(message, function (err) {
        if (err) {
            logger.warn(who, util.format('Failed to queue message to stepTrigger for runId: %s ' +
                'with error: %s', bag.run.id, err));
            return next(true);
        }
        return next();
    });
}
function __normalizeStep(seriesBag, next) {
    var who = seriesBag.who + '|' + __normalizeStep.name;
    logger.trace(who, 'Inside');
    var setup = {};
    if (seriesBag.pipelineStep.ymlConfigPropertyBag &&
        seriesBag.pipelineStep.ymlConfigPropertyBag.runtime)
        setup.runtime = seriesBag.pipelineStep.ymlConfigPropertyBag.runtime;
    if (seriesBag.pipelineStep.ymlConfigPropertyBag &&
        seriesBag.pipelineStep.ymlConfigPropertyBag.environmentVariables)
        setup.environmentVariables =
            seriesBag.pipelineStep.ymlConfigPropertyBag.environmentVariables;
    if (seriesBag.pipelineStep.ymlConfigPropertyBag &&
        seriesBag.pipelineStep.ymlConfigPropertyBag.affinityGroup)
        setup.affinityGroup =
            seriesBag.pipelineStep.ymlConfigPropertyBag.affinityGroup;
    else if (seriesBag.pipeline.ymlConfigPropertyBag &&
        seriesBag.pipeline.ymlConfigPropertyBag.affinityGroup)
        setup.affinityGroup =
            seriesBag.pipeline.ymlConfigPropertyBag.affinityGroup;
    seriesBag.newStep = {
        pipelineId: seriesBag.pipelineStep.pipelineId,
        pipelineSourceId: seriesBag.pipelineStep.pipelineSourceId,
        pipelineStepId: seriesBag.pipelineStep.id,
        projectId: seriesBag.pipelineStep.projectId,
        runId: seriesBag.runId,
        statusCode: global.systemCodesByName.waiting.code,
        configPropertyBag: seriesBag.pipelineStep.ymlConfigPropertyBag,
        execPropertyBag: seriesBag.pipelineStep.ymlExecPropertyBag,
        typeCode: seriesBag.pipelineStep.typeCode,
        name: seriesBag.pipelineStep.name,
        masterResourceId: seriesBag.pipelineStep.masterResourceId || null,
        requiresApproval: !!(seriesBag.pipelineStep.ymlConfigPropertyBag &&
            seriesBag.pipelineStep.ymlConfigPropertyBag.requiresApproval)
    };
    if ((XRAY_ENABLED_STEPS.includes(seriesBag.pipelineStep.yml.type) &&
        seriesBag.pipelineStep.ymlConfigPropertyBag.forceXrayScan === true) ||
        seriesBag.pipelineStep.yml.type === XRAY_SCAN_STEP) {
        seriesBag.newStep.configPropertyBag.xrayDetailsUrl = '';
    }
    if ((seriesBag.inPayload.reset === true) &&
        (seriesBag.pipelineStep.id === seriesBag.inPayload.pipelineStepId)) {
        var configPropertyBag = _.clone(seriesBag.newStep.configPropertyBag);
        configPropertyBag.reset = true;
        seriesBag.newStep.configPropertyBag = configPropertyBag;
    }
    if (_.isEmpty(seriesBag.nodePools) && !seriesBag.isExternalStep)
        seriesBag.skipByNoNodePool = true;
    var innerBag = {
        pipeline: seriesBag.pipeline,
        step: seriesBag.newStep,
        isExternalStep: seriesBag.isExternalStep,
        nodePools: seriesBag.nodePools,
        defaultNodePoolId: seriesBag.defaultNodePoolId,
        globalEnvironmentVariables: seriesBag.globalEnvironmentVariables,
        projectEnvironmentVariables: seriesBag.projectEnvironmentVariables,
        internalIntegrationsById: seriesBag.internalIntegrationsById,
        jFrogPipelinesSessionId: seriesBag.jFrogPipelinesSessionId
    };
    if (seriesBag.pipelineStep.masterResourceId) {
        innerBag.masterResource =
            _.findWhere(seriesBag.masterResources, { id: seriesBag.pipelineStep.masterResourceId });
    }
    if (seriesBag.inPayload.injectedPipeEnvs)
        innerBag.injectedPipeEnvs = seriesBag.inPayload.injectedPipeEnvs;
    if (seriesBag.inPayload.injectedEnvs) {
        innerBag.injectedEnvs = [];
        if (seriesBag.firstPipelineStepId === seriesBag.pipelineStep.id) {
            innerBag.injectedEnvs = seriesBag.inPayload.injectedEnvs
                .filter((injectedEnv) => !injectedEnv.pipelineStepId);
        }
        innerBag.injectedEnvs = innerBag.injectedEnvs.concat(seriesBag.inPayload.injectedEnvs.filter((injectedEnv) => injectedEnv.pipelineStepId === seriesBag.pipelineStep.id));
    }
    if (seriesBag.injectNodePool) {
        innerBag.step.configPropertyBag = {
            ...innerBag.step.configPropertyBag,
            nodePool: seriesBag.injectNodePool
        };
    }
    if (seriesBag.pipeline.ymlConfigPropertyBag?.runtime) {
        seriesBag.pipelineRuntime = seriesBag.pipeline.ymlConfigPropertyBag?.runtime || {};
    }
    if (seriesBag.newStep.configPropertyBag?.runtime) {
        seriesBag.pipelineStepRuntime = seriesBag.newStep.configPropertyBag?.runtime || {};
    }
    normalizeStep(innerBag, function (consoleErrors, resultBag) {
        if (!_.isEmpty(consoleErrors)) {
            logger.warn(util.format('normalizeStep failed for step: %s', seriesBag.pipelineStepId));
            seriesBag.consoleErrors = seriesBag.consoleErrors.concat(consoleErrors);
            return next();
        }
        seriesBag.newStep = resultBag.normalizedStep;
        seriesBag.nodePool = resultBag.nodePool || {};
        return next();
    });
}
function __getOperationResources(seriesBag, next) {
    var who = seriesBag.who + '|' + __getOperationResources.name;
    logger.trace(who, 'Inside');
    var resourceIds = _.compact(_.uniq(_.pluck(seriesBag.connections, 'operationResourceId')));
    if (_.isEmpty(resourceIds)) {
        seriesBag.operationResources = [];
        return next();
    }
    var resourceIdBatches = [];
    var i = 0;
    while (i < resourceIds.length) {
        resourceIdBatches.push(resourceIds.slice(i, i + global.config.defaultBatchSize));
        i += global.config.defaultBatchSize;
    }
    var resources = [];
    async.eachLimit(resourceIdBatches, 10, function (resourceIdBatch, nextBatch) {
        var query = util.format('resourceIds=%s&limit=%s', resourceIdBatch.join(','), global.config.defaultBatchSize);
        seriesBag.suAdapter.getResources(query, function (err, response) {
            if (err) {
                logger.warn(who, util.format('Failed to getResources for query: %s with error: %s', query, err));
                return nextBatch(true);
            }
            resources = resources.concat(response);
            return process.nextTick(nextBatch);
        });
    }, function (err) {
        if (err)
            return next(err);
        seriesBag.operationResources = resources;
        _.each(seriesBag.operationResources, function (operationResource) {
            if (!operationResource.projectIntegrationId)
                return;
            var resInt = seriesBag.projectIntegrationsById[operationResource.projectIntegrationId];
            if (_.isEmpty(resInt))
                seriesBag.consoleErrors.push('Resource ' + operationResource.name +
                    ' is configured to use an integration it does not have ' +
                    'permission to use.');
        });
        return next();
    });
}
function __getMasterResources(bag, next) {
    var who = bag.who + '|' + __getMasterResources.name;
    logger.trace(who, 'Inside');
    var masterResourceIds = _.compact(_.uniq(_.pluck(bag.operationResources, 'masterResourceId')));
    if (bag.pipelineStep.masterResourceId)
        masterResourceIds.push(bag.pipelineStep.masterResourceId);
    if (_.isEmpty(masterResourceIds))
        return next();
    var masterResourceIdBatches = [];
    var i = 0;
    while (i < masterResourceIds.length) {
        masterResourceIdBatches.push(masterResourceIds.slice(i, i + global.config.defaultBatchSize));
        i += global.config.defaultBatchSize;
    }
    var masterResources = [];
    async.eachLimit(masterResourceIdBatches, 10, function (masterResourceIdBatch, nextBatch) {
        var query = util.format('masterResourceIds=%s&limit=%s', masterResourceIdBatch.join(','), global.config.defaultBatchSize);
        bag.suAdapter.getMasterResources(query, function (err, response) {
            if (err) {
                logger.warn(who, util.format('Failed to getMasterResources for query: %s with error: %s', query, err));
                return nextBatch(true);
            }
            masterResources = masterResources.concat(response);
            return process.nextTick(nextBatch);
        });
    }, function (err) {
        if (err)
            return next(err);
        bag.masterResources = masterResources;
        bag.masterResourcesById = _.indexBy(masterResources, 'id');
        return next();
    });
}
function __checkNodePoolCompatibility(bag, next) {
    if (_.isEmpty(bag.masterResources) || _.isEmpty(bag.nodePool))
        return next();
    var who = bag.who + '|' + __checkNodePoolCompatibility.name;
    logger.trace(who, 'Inside');
    var nodePoolOS = bag.nodePool.operatingSystem;
    var nodePoolOsFamily = supportedOsAndFamilyMap[nodePoolOS];
    _.each(bag.masterResources, function (masterResource) {
        var supportedOperatingSystems = _.pluck(masterResource.platformsPropertyBag, 'os');
        if (!supportedOperatingSystems.includes(nodePoolOsFamily)) {
            var msg = util.format('Non-%s resource cannot be executed in %s-type nodepool %s.', nodePoolOsFamily, nodePoolOsFamily, bag.nodePool.name);
            bag.consoleErrors.push(msg);
        }
    });
    return next();
}
function __checkBetaNodePoolCompatibility(bag, next) {
    var who = bag.who + '|' + __checkBetaNodePoolCompatibility.name;
    logger.trace(who, 'Inside');
    if (bag.nodePool?.isBeta && (bag.pipelineStep.yml.type === MATRIX_STEP)) {
        var msg = `Matrix steps are not supported in beta nodepools.`;
        bag.consoleErrors.push(msg);
    }
    return next();
}
function __checkStepIntegrations(seriesBag, next) {
    var who = seriesBag.who + '|' + __checkStepIntegrations.name;
    logger.trace(who, 'Inside');
    var stepIntegrations = seriesBag.pipelineStep.ymlConfigPropertyBag &&
        seriesBag.pipelineStep.ymlConfigPropertyBag.integrations;
    _.each(stepIntegrations, function (int) {
        if (_.isEmpty(seriesBag.projectIntegrationsByName[int.name]))
            seriesBag.consoleErrors.push('Step ' + seriesBag.pipelineStep.name +
                ' does not have permission to use integration ' + int.name + '.');
    });
    return next();
}
function __getOperationResourceVersions(seriesBag, next) {
    var who = seriesBag.who + '|' + __getOperationResourceVersions.name;
    logger.trace(who, 'Inside');
    async.eachLimit(seriesBag.operationResources, 10, function (resource, done) {
        if (!resource.latestResourceVersionId)
            return process.nextTick(done);
        let versionId = resource.latestResourceVersionId;
        let isPinnedVersion = false;
        if (seriesBag.triggersByPipelineStepId &&
            seriesBag.triggersByPipelineStepId[seriesBag.pipelineStep.id] &&
            seriesBag.triggersByPipelineStepId[seriesBag.pipelineStep.id].
                resourceId === resource.id) {
            versionId = seriesBag.triggersByPipelineStepId[seriesBag.pipelineStep.id].resourceVersionId;
        }
        let resourceOverrideVersion;
        if (seriesBag.inPayload.overrideVersions) {
            let overrideVersion = seriesBag.inPayload.overrideVersions.find((version) => version.pipelineStepId === seriesBag.pipelineStep.id &&
                version.resourceId === resource.id);
            if (!overrideVersion &&
                seriesBag.firstPipelineStepId === seriesBag.pipelineStep.id) {
                overrideVersion = seriesBag.inPayload.overrideVersions.find((version) => (version.resourceId === resource.id && !version.pipelineStepId));
            }
            if (overrideVersion && overrideVersion.resourceVersionId) {
                resourceOverrideVersion = overrideVersion.resourceVersionId;
            }
        }
        if (!resourceOverrideVersion && resource.pinnedVersionId) {
            isPinnedVersion = true;
            resourceOverrideVersion = resource.pinnedVersionId;
        }
        if (resourceOverrideVersion)
            versionId = resourceOverrideVersion;
        seriesBag.suAdapter.getResourceVersionById(versionId, function (err, version) {
            if (err) {
                logger.warn(who, util.format('Failed to getResourceVersionById for id: %s with error: %s', versionId, err));
            }
            if (version && resource.id === version.resourceId)
                resource.version = version;
            else {
                let errorMessage;
                if (isPinnedVersion)
                    errorMessage = 'Pinned resource version id %s not found for ' +
                        'resource %s';
                else
                    errorMessage = 'Resource version id %s not found for resource ' +
                        '%s';
                seriesBag.consoleErrors.push(util.format(errorMessage, versionId, resource.name));
                resource.version = {};
            }
            return process.nextTick(done);
        });
    }, function () {
        return next();
    });
}
function __getHelloWorldTemplate(seriesBag, next) {
    const who = seriesBag.who + '|' + __getHelloWorldTemplate.name;
    logger.trace(who, 'Inside');
    const query = util.format('names=%s&limit=10', HELLO_WORLD_TEMPLATE_NAME);
    seriesBag.suAdapter.getTemplates(query, function (err, templates) {
        if (err || _.isEmpty(templates)) {
            logger.warn(who, 'Failed to get HelloWorld template', err);
            return next(true);
        }
        seriesBag.helloWorldTemplate = templates.find((template) => template.namespace === JFROG_TEMPLATE_NAMESPACE);
        return next();
    });
}
function __createStep(seriesBag, next) {
    var who = seriesBag.who + '|' + __createStep.name;
    logger.trace(who, 'Inside');
    seriesBag.newStep.triggeredAt = new Date();
    if (seriesBag.pipelineStep.ymlConfigPropertyBag &&
        seriesBag.pipelineStep.ymlConfigPropertyBag.affinityGroup)
        seriesBag.newStep.affinityGroup =
            seriesBag.pipelineStep.ymlConfigPropertyBag.affinityGroup;
    else if (seriesBag.pipeline.ymlConfigPropertyBag &&
        seriesBag.pipeline.ymlConfigPropertyBag.affinityGroup)
        seriesBag.newStep.affinityGroup =
            seriesBag.pipeline.ymlConfigPropertyBag.affinityGroup;
    if (seriesBag.triggersByPipelineStepId &&
        seriesBag.triggersByPipelineStepId[seriesBag.newStep.pipelineStepId])
        seriesBag.newStep.triggeredByResourceVersionId =
            seriesBag.triggersByPipelineStepId[seriesBag.newStep.pipelineStepId].resourceVersionId;
    if (seriesBag.triggeredByIdentity)
        seriesBag.newStep.triggeredByIdentityId =
            seriesBag.triggeredByIdentity.id;
    const internalBuildIntegration = seriesBag.projectIntegrationsByName[ConfigFactory.config.getCustom('runtime.internalBuildIntegrationName')];
    const isRestrictedRaw = ConfigFactory.config.getCustom('runtime.isRestricted');
    const isRestricted = isRestrictedRaw === true || isRestrictedRaw === 'true';
    const isHelloWorldPipeline = seriesBag.pipelineSource.templateId === seriesBag.helloWorldTemplate.id;
    if (isRestricted &&
        !_.isEmpty(internalBuildIntegration) &&
        seriesBag.nodePool.projectIntegrationId ===
            internalBuildIntegration.id &&
        !isHelloWorldPipeline) {
        seriesBag.skipByRestrictedMode = true;
    }
    if (seriesBag.skipByNoNodePool || seriesBag.skipByRestrictedMode) {
        seriesBag.newStep.statusCode = global.systemCodesByName.skipped.code;
        seriesBag.newStep.pendingLogsComplete = true;
    }
    else if (!_.isEmpty(seriesBag.consoleErrors)) {
        seriesBag.newStep.statusCode = global.systemCodesByName.error.code;
        seriesBag.newStep.pendingLogsComplete = true;
    }
    else if (seriesBag.isExternalStep) {
        if (!_.isEmpty(seriesBag.payloadConfigPropertyBag)) {
            seriesBag.newStep.statusCode = global.systemCodesByName.processing.code;
            seriesBag.newStep.groupInProgress = true;
            seriesBag.newStep.configPropertyBag = _.extend(seriesBag.newStep.configPropertyBag, seriesBag.payloadConfigPropertyBag);
        }
        if (seriesBag.externalBuildId)
            seriesBag.newStep.externalBuildId = seriesBag.externalBuildId;
        if (seriesBag.externalBuildUrl)
            seriesBag.newStep.externalBuildUrl = seriesBag.externalBuildUrl;
    }
    seriesBag.suAdapter.postStep(seriesBag.newStep, function (err, step) {
        if (err) {
            logger.warn(who, util.format('Failed to postStep for pipelineStepId: %s with error: %s', seriesBag.pipelineStepId, err));
            return next(true);
        }
        seriesBag.step = step;
        return next();
    });
}
function __createSteplets(seriesBag, next) {
    var who = seriesBag.who + '|' + __createSteplets.name;
    logger.trace(who, 'Inside');
    var bag = {
        who: who,
        jFrogPipelinesSessionId: seriesBag.jFrogPipelinesSessionId,
        step: seriesBag.step,
        pipelineRuntime: seriesBag.pipelineRuntime,
        pipelineStepRuntime: seriesBag.pipelineStepRuntime,
        pipelineStep: seriesBag.pipelineStep,
        pipelineSteplets: [],
        suAdapter: seriesBag.suAdapter,
        skipByNoNodePool: seriesBag.skipByNoNodePool,
        skipByRestrictedMode: seriesBag.skipByRestrictedMode,
        consoleErrors: seriesBag.consoleErrors || [],
        stepletErrorsByPipelineStepletId: {},
        nodePools: seriesBag.nodePools,
        nodePool: seriesBag.nodePool,
        steplets: []
    };
    async.series([
        ___getPipelineSteplets.bind(null, bag),
        ___normalizePipelineSteplets.bind(null, bag),
        ___createSteplets.bind(null, bag),
        ___postStepletConsoles.bind(null, bag),
        ___updateStepStatus.bind(null, bag)
    ], function (err) {
        seriesBag.steplets = bag.steplets;
        return next(err);
    });
}
function ___getPipelineSteplets(bag, next) {
    var who = bag.who + '|' + ___getPipelineSteplets.name;
    logger.trace(who, 'Inside');
    var query = util.format('pipelineStepIds=%s&isDeleted=false', bag.pipelineStep.id);
    bag.suAdapter.getPipelineSteplets(query, function (err, pipelineSteplets) {
        if (err) {
            var msg = util.format('getPipelineSteplets failed for pipelineStep id: %s with error: %s', bag.pipelineStep.id, err.message);
            logger.warn(msg);
            return next(true);
        }
        bag.pipelineSteplets = pipelineSteplets;
        return next();
    });
}
function ___normalizePipelineSteplets(bag, next) {
    if (_.isEmpty(bag.pipelineSteplets))
        return next();
    if (bag.skipByNoNodePool || _.isEmpty(bag.nodePool))
        return next();
    var who = bag.who + '|' + ___normalizePipelineSteplets.name;
    logger.trace(who, 'Inside');
    async.eachLimit(bag.pipelineSteplets, 10, function (pipelineSteplet, nextPipelineSteplet) {
        var normalizeStepletBag = {
            consoleErrors: bag.consoleErrors,
            stepletErrors: [],
            pipelineSteplet: pipelineSteplet,
            step: bag.step,
            pipelineRuntime: bag.pipelineRuntime,
            pipelineStepRuntime: bag.pipelineStepRuntime,
            nodePool: bag.nodePool,
            nodePools: bag.nodePools
        };
        async.series([
            ____normalizeStepletAffinityGroup.bind(null, normalizeStepletBag),
            ____normalizeStepletNodePool.bind(null, normalizeStepletBag),
            ____normalizeStepletConfigRuntimeSection.bind(null, normalizeStepletBag)
        ], function (err) {
            bag.stepletErrorsByPipelineStepletId[pipelineSteplet.id] =
                normalizeStepletBag.stepletErrors;
            return nextPipelineSteplet(err);
        });
    }, function (err) {
        return next(err);
    });
}
function ___createSteplets(bag, next) {
    var who = bag.who + '|' + ___createSteplets.name;
    logger.trace(who, 'Inside');
    const defaultPriority = 9999;
    const externalStepTypes = ['Jenkins'];
    const stepletConfigPropertyBag = {
        nodePool: bag.step?.configPropertyBag?.nodePool,
        isExternal: _.contains(externalStepTypes, global.systemCodesByCode[bag.step.typeCode].name)
    };
    var defaultSteplet = {
        projectId: bag.pipelineStep.projectId,
        pipelineId: bag.pipelineStep.pipelineId,
        pipelineSourceId: bag.pipelineStep.pipelineSourceId,
        stepId: bag.step.id,
        stepletNumber: 1,
        priority: bag.step.configPropertyBag &&
            bag.step.configPropertyBag.priority || defaultPriority,
        affinityGroup: bag.step.affinityGroup,
        configPropertyBag: stepletConfigPropertyBag,
        statusCode: bag.step.statusCode,
        triggeredAt: new Date()
    };
    var stepletsToCreate = [];
    if (_.isEmpty(bag.pipelineSteplets)) {
        stepletsToCreate.push(defaultSteplet);
    }
    else {
        _.each(bag.pipelineSteplets, function (pipelineSteplet) {
            var newSteplet = {
                projectId: bag.pipelineStep.projectId,
                pipelineId: bag.pipelineStep.pipelineId,
                pipelineSourceId: bag.pipelineStep.pipelineSourceId,
                stepId: bag.step.id,
                stepletNumber: pipelineSteplet.configPropertyBag.stepletNumber,
                configPropertyBag: {
                    ...stepletConfigPropertyBag,
                    ...pipelineSteplet.configPropertyBag
                },
                statusCode: bag.step.statusCode,
                priority: bag.step.configPropertyBag &&
                    bag.step.configPropertyBag.priority || defaultPriority,
                affinityGroup: pipelineSteplet.affinityGroup,
                triggeredAt: new Date()
            };
            if (!_.isEmpty(bag.stepletErrorsByPipelineStepletId[pipelineSteplet.id]))
                newSteplet.statusCode = global.systemCodesByName.error.code;
            stepletsToCreate.push(newSteplet);
        });
    }
    async.eachLimit(stepletsToCreate, 10, function (steplet, nextSteplet) {
        bag.suAdapter.postSteplet(steplet, function (err, createdSteplet) {
            if (err) {
                logger.warn(bag.who, util.format('Failed to postSteplet for stepId: %s with error: %s', bag.step.id, err));
                return nextSteplet(true);
            }
            bag.steplets.push(createdSteplet);
            return process.nextTick(nextSteplet);
        });
    }, function (err) {
        return next(err);
    });
}
function ___postStepletConsoles(bag, next) {
    var who = bag.who + '|' + ___postStepletConsoles.name;
    logger.trace(who, 'Inside');
    const stepletByNumber = _.indexBy(bag.steplets, 'stepletNumber');
    async.eachLimit(bag.pipelineSteplets, 10, function (pipelineSteplet, nextSteplet) {
        var stepletConsoles = bag.stepletErrorsByPipelineStepletId[pipelineSteplet.id];
        if (_.isEmpty(stepletConsoles))
            return process.nextTick(nextSteplet);
        const steplet = stepletByNumber[pipelineSteplet.configPropertyBag.stepletNumber];
        if (!steplet)
            return process.nextTick(nextSteplet);
        var consoleAdapter = new BatchStepletConsoleAdapter(bag.jFrogPipelinesSessionId, global.config.apiToken, [steplet.id], steplet.pipelineId, global.systemConfig.jobConsoleBatchSize, global.systemConfig.jobConsoleBufferTimeInterval);
        var grpMsg = 'ERROR';
        var logMessage = stepletConsoles.join('\n');
        consoleAdapter.openGrp(grpMsg);
        consoleAdapter.openCmd('Info');
        consoleAdapter.publishMsg(logMessage);
        consoleAdapter.closeCmd(false);
        consoleAdapter.closeGrp(false);
        return process.nextTick(nextSteplet);
    }, function (err) {
        return next(err);
    });
}
function ___updateStepStatus(bag, next) {
    var nonErrorStepletStatuses = _.without(_.pluck(bag.steplets, 'statusCode'), global.systemCodesByName.error.code);
    if (!_.isEmpty(nonErrorStepletStatuses))
        return next();
    var who = bag.who + '|' + ___updateStepStatus.name;
    logger.trace(who, 'Inside');
    var update = {
        statusCode: global.systemCodesByName.error.code
    };
    bag.suAdapter.putStepById(bag.step.id, update, function (err, updatedStep) {
        if (err) {
            logger.warn(bag.who, util.format('Failed to putStepById for stepId: %s with error: %s', bag.step.id, util.inspect(err)));
            return next(true);
        }
        bag.step.statusCode = updatedStep.statusCode;
        return next();
    });
}
function ____normalizeStepletConfigRuntimeSection(bag, next) {
    var who = bag.who + '|' + ____normalizeStepletConfigRuntimeSection.name;
    logger.trace(who, 'Inside');
    if (_.isEmpty(bag.pipelineSteplet.configPropertyBag))
        return next();
    if (_.isEmpty(bag.pipelineSteplet.configPropertyBag.runtime)
        && _.isEmpty(bag.pipelineSteplet.configPropertyBag.nodePool))
        return next();
    if (bag.stepletErrors.length > 0)
        return next();
    let pipelineRuntime = bag.pipelineRuntime;
    let stepRuntime = bag.pipelineStepRuntime;
    let pipelineStepletRuntime = bag.pipelineSteplet?.configPropertyBag?.runtime;
    let configuredRuntime = pipelineStepletRuntime || stepRuntime || pipelineRuntime || {};
    var runtime = {
        type: configuredRuntime.type,
        image: configuredRuntime.image || {}
    };
    var os = bag.nodePool.operatingSystem;
    if (!runtime.type) {
        if (os.startsWith('MacOS'))
            runtime.type = 'host';
        else
            runtime.type = 'image';
    }
    if (runtime.type !== 'host') {
        var imageName = '';
        var imageTag = '';
        var autoPull = true;
        var containerOptions = '';
        var image = runtime.image || {};
        var arch = bag.nodePool.architecture;
        if (os.startsWith('MacOS')) {
            bag.stepletErrors.push('The selected node pool contains MacOS nodes. ' +
                'Steps on a MacOS node must have runtime type host.');
            return next();
        }
        if (_.isEmpty(image.custom) && !(image.auto && image.auto.language)) {
            var stepType = _.findWhere(global.systemCodes, {
                code: bag.step.typeCode,
                group: 'resource'
            }).name;
            if (stepType === 'GoBuild' ||
                stepType === 'GoPublishBinary' || stepType === 'GoPublishModule') {
                image.auto = image.auto || {};
                image.auto.language = 'go';
            }
            else if (stepType === 'GradleBuild') {
                image.auto = image.auto || {};
                image.auto.language = 'java';
            }
            else if (stepType === 'MvnBuild') {
                image.auto = image.auto || {};
                image.auto.language = 'java';
            }
            else if (stepType === 'NpmBuild' || stepType === 'NpmPublish') {
                image.auto = image.auto || {};
                image.auto.language = 'node';
            }
        }
        if (!_.isEmpty(image.custom)) {
            var custom = image.custom;
            imageName = custom.name || '';
            imageTag = custom.tag || '';
            autoPull = _.isBoolean(custom.autoPull) ? custom.autoPull : true;
            containerOptions = custom.options || '';
            runtime.image = {
                imageName: imageName,
                imageTag: imageTag,
                containerOptions: containerOptions,
                autoPull: autoPull
            };
            if (custom.registry)
                runtime.image.registry = custom.registry;
            if (custom.sourceRepository)
                runtime.image.sourceRepository = custom.sourceRepository;
            if (custom.region)
                runtime.image.region = custom.region;
        }
        else if (image.auto && image.auto.language) {
            var language = image.auto.language.toString().toLowerCase();
            var langImage = _.find(global.config.languageImages, function (configImage) {
                return configImage.architecture === arch && configImage.os === os &&
                    configImage.language &&
                    configImage.language.toString().toLowerCase() === language;
            });
            imageName = langImage && langImage.image;
            if (_.isEmpty(imageName)) {
                bag.stepletErrors.push('No default image has been configured for ' +
                    'architecture: ' + arch + ', operating system: ' + os +
                    ', and language: ' + image.auto.language + '. ' +
                    'Please add a custom image to the runtime configuration for this ' +
                    'step in your YML file or see the documentation to configure a ' +
                    'default docker image for this language at the system level.');
                return next();
            }
            if (langImage.registryUrl)
                imageName = langImage.registryUrl + '/' + imageName;
            imageTag = '';
            if (!_.isEmpty(image.auto.versions))
                imageTag = image.auto.versions[0];
            else if (langImage.defaultVersion)
                imageTag = langImage.defaultVersion;
            runtime.image = {
                imageName: imageName,
                imageTag: imageTag
            };
        }
        else {
            var defaultLangImage = _.findWhere(global.config.languageImages, {
                architecture: arch,
                os: os,
                isDefault: true
            });
            imageName = defaultLangImage && defaultLangImage.image;
            if (_.isEmpty(imageName)) {
                bag.stepletErrors.push('No default image has been configured for ' +
                    'architecture: ' + arch + ' and operating system: ' + os + '. ' +
                    'Please add a custom image to the runtime configuration for this ' +
                    'step in your YML file or see the documentation to configure a ' +
                    'default docker image at the system level.');
                return next();
            }
            if (defaultLangImage.registryUrl)
                imageName = defaultLangImage.registryUrl + '/' + imageName;
            var defaultLanguageImageName = imageName;
            var defaultLanguageImageVersion = defaultLangImage &&
                defaultLangImage.defaultVersion || '';
            runtime.image = {
                imageName: defaultLanguageImageName,
                imageTag: defaultLanguageImageVersion
            };
        }
        pipelineStepletRuntime = _.extend({}, pipelineStepletRuntime, runtime);
    }
    else {
        pipelineStepletRuntime = {
            type: 'host'
        };
    }
    bag.pipelineSteplet.configPropertyBag.runtime = pipelineStepletRuntime;
    return next();
}
function __createRunResourceVersions(seriesBag, next) {
    var who = seriesBag.who + '|' + __createRunResourceVersions.name;
    logger.trace(who, 'Inside');
    async.eachLimit(seriesBag.operationResources, 10, function (resource, done) {
        if (!resource.version || !resource.version.id)
            return process.nextTick(done);
        var newResourceVersion = {
            projectId: seriesBag.pipelineStep.projectId,
            pipelineSourceId: seriesBag.pipelineStep.pipelineSourceId,
            pipelineSourceBranch: resource.pipelineSourceBranch,
            pipelineId: seriesBag.pipelineStep.pipelineId,
            runId: seriesBag.runId,
            stepId: seriesBag.step.id,
            resourceName: resource.name,
            resourceTypeCode: resource.typeCode,
            resourceVersionId: resource.version.id,
            resourceStaticPropertyBag: resource.staticPropertyBag,
            resourceConfigPropertyBag: resource.ymlConfigPropertyBag,
            resourceVersionContentPropertyBag: resource.version.contentPropertyBag,
            resourceVersionCreatedByStepId: resource.version.createdByStepId,
            masterResourceId: resource.masterResourceId
        };
        var isGitRepoResource = false;
        if (resource.typeCode === global.systemCodesByName.GitRepo.code)
            isGitRepoResource = true;
        newResourceVersion.resourceVersionContentPropertyBag =
            newResourceVersion.resourceVersionContentPropertyBag || {};
        if (isGitRepoResource &&
            _.has(global.config, 'autoSyncResourceIfOutdated'))
            newResourceVersion.resourceVersionContentPropertyBag.
                autoSyncResourceIfOutdated = global.config.autoSyncResourceIfOutdated;
        seriesBag.suAdapter.postRunResourceVersion(newResourceVersion, function (err, runResourceVersion) {
            if (err) {
                seriesBag.consoleErrors.push('Failed to create required ' +
                    `version for ${resource.name} resource `);
                logger.error(who, util.format('Failed to postRunResourceVersion for resourceId: %s ' +
                    'with error: %s', resource.id, err));
                return done(err);
            }
            resource.runResourceVersion = runResourceVersion;
            runResourceVersion.resourceId = resource.id;
            seriesBag.runResourceVersions.push(runResourceVersion);
            return process.nextTick(done);
        });
    }, function (err) {
        if (err) {
            logger.error('Failed to publish resource version, ' +
                'will mark step as Error');
            seriesBag.isPostingResourceVersionErr = true;
        }
        return next();
    });
}
function __errorOutStepIfResourceVersionPublishFailed(seriesBag, next) {
    if (!seriesBag.isPostingResourceVersionErr)
        return next();
    let who = seriesBag.who + '|' +
        __errorOutStepIfResourceVersionPublishFailed.name;
    logger.trace(who, 'Inside');
    let update = {
        statusCode: global.systemCodesByName.error.code
    };
    seriesBag.step.statusCode = global.systemCodesByName.error.code;
    seriesBag.suAdapter.putStepById(seriesBag.step.id, update, function (err) {
        if (err) {
            logger.warn(who, util.format('Failed to putStepById for stepId: %s with error: %s', seriesBag.step.id, util.inspect(err)));
            return next(true);
        }
    });
    return next();
}
function __createStepJson(seriesBag, next) {
    if (!global.config.signedPipelinesEnabled)
        return next();
    var who = seriesBag.who + '|' + __createStepJson.name;
    logger.trace(who, 'Inside');
    const payload = {
        step: seriesBag.step,
        runResourceVersions: seriesBag.runResourceVersions,
        projectIntegrationsByName: seriesBag.projectIntegrationsByName,
        masterIntegrationsById: seriesBag.masterIntegrationsById,
        masterResourcesById: seriesBag.masterResourcesById,
        pipelineStepsByName: seriesBag.pipelineStepsByName,
        nodePool: seriesBag.nodePool,
        triggeredByIdentity: seriesBag.triggeredByIdentity,
        pipeline: seriesBag.pipeline,
        pipelineSource: seriesBag.pipelineSource,
        runNumber: seriesBag.runNumber,
        jfrogPipelinesSessionId: seriesBag.jFrogPipelinesSessionId
    };
    const createStepJson = new createStepJson_1.CreateStepJson(payload);
    createStepJson.createPipeInfo()
        .catch(error => {
        logger.warn(util.format('createStepJson failed for step: %s', bag.step.id));
    }).finally(() => {
        next();
    });
}
function __postStepConsoles(seriesBag, next) {
    if (!(seriesBag.skipByNoNodePool || seriesBag.skipByRestrictedMode
        || !_.isEmpty(seriesBag.consoleErrors))) {
        return next();
    }
    var who = seriesBag.who + '|' + __postStepConsoles.name;
    logger.trace(who, 'Inside');
    var consoleAdapter = new StepConsoleAdapter(seriesBag.jFrogPipelinesSessionId, global.config.apiToken, seriesBag.step.id, seriesBag.step.pipelineId, global.systemConfig.jobConsoleBatchSize, global.systemConfig.jobConsoleBufferTimeInterval);
    var grpMsg;
    var logMessage;
    var closeCmd = false;
    var closeGrp = false;
    if (seriesBag.skipByNoNodePool) {
        grpMsg = 'Step skipped';
        logMessage = 'Skipping step as no nodePool is found.';
    }
    else if (seriesBag.skipByRestrictedMode) {
        grpMsg = 'Step skipped';
        logMessage = 'Your Free Tier subscription only allows execution of the' +
            ' hello_world pipeline on JFrog hosted nodes. ' +
            'To run custom pipelines, you can do one of the following: \n' +
            '• Validate your identity using a credit card by visiting https://my.jfrog.com/quick-purchase/checkout. Your credit card will not be charged.\n' +
            '• Add custom nodes by following the instructions here: https://www.jfrog.com/confluence/display/JFROG/Managing+Pipelines+Node+Pools.\n' +
            '• Upgrade your subscription to a paid plan: https://jfrog.com/pricing/';
    }
    else if (!_.isEmpty(seriesBag.consoleErrors)) {
        grpMsg = 'ERROR';
        logMessage = seriesBag.consoleErrors.join('\n');
    }
    consoleAdapter.openGrp(grpMsg);
    consoleAdapter.openCmd('Info');
    consoleAdapter.publishMsg(logMessage);
    consoleAdapter.closeCmd(closeCmd);
    consoleAdapter.closeGrp(closeGrp);
    return next();
}
function ____normalizeStepletNodePool(bag, next) {
    var who = bag.who + '|' + ____normalizeStepletNodePool.name;
    logger.trace(who, 'Inside');
    let stepletPropertyBag = bag.pipelineSteplet.configPropertyBag;
    if (stepletPropertyBag && stepletPropertyBag.nodePool) {
        let availablePools = bag.nodePools.filter(function (pool) {
            return pool.name === stepletPropertyBag.nodePool;
        });
        if (availablePools.length === 0) {
            let message = 'No Node Pool with name' + stepletPropertyBag.nodePool + ' found';
            bag.stepletErrors.push(message);
            return next();
        }
        if (availablePools.length > 0) {
            bag.nodePool = availablePools[0];
        }
    }
    next();
}
function ____normalizeStepletAffinityGroup(bag, next) {
    var who = bag.who + '|' + ____normalizeStepletAffinityGroup.name;
    logger.trace(who, 'Inside');
    if (bag.step.configPropertyBag && bag.step.configPropertyBag.multiNode === true) {
        bag.pipelineSteplet.affinityGroup =
            `${bag.step.affinityGroup}_${bag.pipelineSteplet?.configPropertyBag?.stepletNumber}`;
    }
    else {
        bag.pipelineSteplet.affinityGroup = `${bag.step.affinityGroup}`;
    }
    return next();
}
//# sourceMappingURL=createRun.js.map