"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const createInfoJson_1 = require("../createInfoJson");
jest.mock('../../../acl/syncService/SyncRepository', () => {
    return {
        SyncRepository: jest.fn().mockReturnValue({
            postPipelineObjects: function (pipelineObjectBody) {
                return Promise.resolve(pipelineObjectBody);
            }
        })
    };
});
const mockRun = function () {
    return {
        staticPropertyBag: {
            triggeredByUserName: 'admin',
            signedPipelinesEnabled: true
        },
        systemPropertyBag: '<systemPropertyBag>',
        id: 1,
        name: '<name>',
        pipelineId: 1,
        pipelineSourceId: 1,
        projectId: 1,
        parentRunId: 1,
        runNumber: 1,
        durationSeconds: 2,
        statusCode: 4015,
        description: '<description>',
        pubKey: '<pubKey>',
        merkleLeaves: [],
        merkleRoot: '<merkleRoot>',
        endedAt: new Date('2023-06-20T11:48:53.943Z'),
        startedAt: new Date('2023-06-20T11:48:53.798Z'),
        triggerId: 1
    };
};
const mockPipeline = function () {
    return {
        ymlConfigPropertyBag: {},
        ymlPropertyBag: {
            name: 'intro_hello_world',
            steps: [
                {
                    name: 'first',
                    type: 'Bash',
                    execution: {
                        onExecute: ['jfrog --version']
                    }
                }
            ]
        },
        id: 1,
        name: 'intro_hello_world',
        pipelineSourceBranch: '<pipelineSourceBranch>',
        projectId: 1,
        pipelineSourceId: 1,
        latestRunNumber: 266,
        latestRunId: 266,
        latestSuccessfulRunId: 0,
        latestFailedRunId: 0,
        latestErrorRunId: 0,
        latestSkippedRunId: 0,
        latestTimedOutRunId: 0,
        latestCancelledRunId: 0,
        latestUnstableRunId: 0,
        latestCompletedRunId: 0,
        syntaxVersion: 'v1.0',
        isDeleted: false,
        deletedAt: new Date('2023-06-20T11:47:40.660Z')
    };
};
const mockPipelineSources = function () {
    return [
        {
            valuesYmlPropertyBag: {
                namePrefix: 'intro'
            },
            pipelinesYmlPropertyBag: {},
            id: 1,
            name: 'hello_world',
            projectId: 1,
            projectIntegrationId: 0,
            repositoryFullName: '<repositoryFullName>',
            isMultiBranch: false,
            isInternal: true,
            branch: '<branch>',
            branchExcludePattern: '',
            branchIncludePattern: '',
            fileFilter: '',
            environments: [],
            isSyncing: false,
            isDeleting: false,
            lastSyncStatusCode: 4002,
            lastSyncStartedAt: new Date('2023-06-20T11:47:38.636Z'),
            lastSyncEndedAt: new Date('2023-06-20T11:47:41.029Z'),
            lastSyncLogs: '<lastSyncLogs>',
            resourceId: 0,
            createdBy: 1,
            updatedBy: 1,
            templateId: 9,
            accessResourceNamePrefix: ''
        },
        {
            valuesYmlPropertyBag: {
                namePrefix: 'intro'
            },
            pipelinesYmlPropertyBag: {},
            id: 2,
            name: 'hello_world',
            projectId: 1,
            projectIntegrationId: 0,
            repositoryFullName: '<repositoryFullName>',
            isMultiBranch: true,
            isInternal: true,
            branch: '<branch>',
            branchExcludePattern: '',
            branchIncludePattern: '',
            fileFilter: '',
            environments: [],
            isSyncing: false,
            isDeleting: false,
            lastSyncStatusCode: 4002,
            lastSyncStartedAt: new Date('2023-06-20T11:47:38.636Z'),
            lastSyncEndedAt: new Date('2023-06-20T11:47:41.029Z'),
            lastSyncLogs: '<lastSyncLogs>',
            resourceId: 0,
            createdBy: 1,
            updatedBy: 1,
            templateId: 9,
            accessResourceNamePrefix: ''
        }
    ];
};
describe('CreateInfoJson', () => {
    const jFrogPipelinesSessionId = '<jFrogPipelinesSessionId>';
    it('throws error for empty run', async () => {
        const createInfoJson = new createInfoJson_1.CreateInfoJson({
            jFrogPipelinesSessionId,
            pipeline: mockPipeline(),
            pipelineSource: mockPipelineSources().find((source) => !!source.isMultiBranch)
        });
        await expect(createInfoJson.createPipeInfo()).rejects.toThrow('Run is empty');
    });
    it('throws error for empty pipeline', async () => {
        const createInfoJson = new createInfoJson_1.CreateInfoJson({
            jFrogPipelinesSessionId,
            run: mockRun()
        });
        await expect(createInfoJson.createPipeInfo()).rejects.toThrow('Pipeline is empty');
    });
    it('pass for proper payload without multibranch', async () => {
        const createInfoJson = new createInfoJson_1.CreateInfoJson({
            jFrogPipelinesSessionId,
            run: mockRun(),
            pipeline: mockPipeline(),
            pipelineSource: mockPipelineSources().find((source) => !!source.isMultiBranch)
        });
        const result = await createInfoJson.createPipeInfo();
        expect(result).toBeDefined();
    });
    it('pass for proper payload with multibranch', async () => {
        const createInfoJson = new createInfoJson_1.CreateInfoJson({
            jFrogPipelinesSessionId,
            run: mockRun(),
            pipeline: mockPipeline(),
            pipelineSource: mockPipelineSources().find((source) => !source.isMultiBranch)
        });
        const result = await createInfoJson.createPipeInfo();
        expect(result).toBeDefined();
    });
});
//# sourceMappingURL=createInfoJson.test.js.map