"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const createStepJson_1 = require("../createStepJson");
jest.mock('../../../acl/syncService/SyncRepository', () => {
    return {
        SyncRepository: jest.fn().mockReturnValue({
            postPipelineObjects: function (pipelineObjectBody) {
                return Promise.resolve(pipelineObjectBody);
            }
        })
    };
});
const mockStep = function () {
    return {
        configPropertyBag: {
            requiresApproval: false,
            integrations: [
                {
                    name: 'githubEnterprise'
                }
            ],
            inputSteps: [
                {
                    name: 'add_run_variable'
                },
                {
                    name: 'variable_selection'
                }
            ],
            outputResources: [
                {
                    name: 'intro_bag'
                }
            ],
            environmentVariables: [],
            nodePool: 'oneboxMacOS12',
            timeoutSeconds: 3600,
            runtime: {
                type: 'host'
            }
        },
        execPropertyBag: {
            onExecute: ['echo "onExecute"']
        },
        systemPropertyBag: {},
        id: 1,
        pipelineId: 1,
        pipelineSourceId: 1,
        name: 'write_to_resource',
        pipelineStepId: 5,
        projectId: 1,
        runId: 1,
        statusCode: 4005,
        masterResourceId: 1,
        groupInProgress: false,
        typeCode: 2007,
        affinityGroup: 'write_to_resource',
        pipelineStateArtifactName: 'null',
        triggeredByIdentityId: 3,
        triggeredAt: new Date('2023-06-21T12:10:53.532Z'),
        requiresApproval: false,
        triggeredByResourceVersionId: 1,
        triggeredByStepId: 0,
        staticPropertyBag: {}
    };
};
const mockRunResourceVersions = function () {
    return [
        {
            resourceStaticPropertyBag: {},
            resourceConfigPropertyBag: {},
            id: 1,
            stepId: 1,
            projectId: 1,
            pipelineSourceId: 1,
            pipelineSourceBranch: '<pipelineSourceBranch>',
            runId: 1,
            pipelineId: 1,
            resourceTypeCode: 1034,
            resourceVersionId: 1,
            resourceName: 'intro_bag',
            resourceVersionCreatedByStepId: 0,
            masterResourceId: 0
        }
    ];
};
const mockTriggeredByIdentity = function () {
    return {
        id: 3,
        name: 'admin',
        type: 6210
    };
};
const mockPipelineSource = function () {
    return {
        valuesYmlPropertyBag: {
            namePrefix: 'intro'
        },
        pipelinesYmlPropertyBag: {},
        id: 1,
        name: 'hello_world',
        projectId: 1,
        projectIntegrationId: 0,
        repositoryFullName: '<repositoryFullName>',
        isMultiBranch: false,
        isInternal: true,
        branch: '<branch>',
        branchExcludePattern: '<branchExcludePattern>',
        branchIncludePattern: '<branchIncludePattern>',
        fileFilter: 'fileFilter',
        environments: [],
        isSyncing: false,
        isDeleting: false,
        lastSyncStatusCode: 4002,
        resourceId: 0,
        createdBy: 1,
        updatedBy: 1,
        templateId: 9
    };
};
const mockNodePool = function () {
    return {
        providerMetadataPropertyBag: {},
        cacheSettingsPropertyBag: {},
        initPropertyBag: {},
        permissions: {
            allowAllPipelineSources: true
        },
        id: 1,
        projectId: 1,
        environments: [],
        name: 'oneboxMacOS12',
        queueName: '<queueName>',
        numberOfNodes: 1,
        isOnDemand: false,
        architecture: 'ARM64',
        operatingSystem: 'MacOS_12',
        maxDiskUsagePercentage: 90,
        diskSizeinGB: 60,
        timeoutMS: 1000,
        isNatEnabled: false,
        isCacheEnabled: false,
        initAsNonRoot: false,
        projectIntegrationId: 0,
        buildPlaneImageId: 0,
        nodeIdleIntervalInMins: 10,
        accessResourceNamePrefix: ''
    };
};
const mockPipelineStepsByName = function () {
    return {
        addRunVariable: {
            ymlConfigPropertyBag: {
                inputSteps: [
                    {
                        name: 'first'
                    }
                ],
                environmentVariables: {
                    normal: {}
                }
            },
            ymlExecPropertyBag: {
                onStart: ['echo "This step will save a value to a run variable."'],
                onExecute: [
                    'add_run_variables step_info="runNumber $run_number, step_name $step_name, pipeline_name $pipeline_name"'
                ]
            },
            yml: {
                name: 'add_run_variable',
                type: 'Bash',
                configuration: {
                    inputSteps: [
                        {
                            name: 'first'
                        }
                    ]
                },
                execution: {
                    onStart: ['echo "This step will save a value to a run variable."'],
                    onExecute: [
                        'add_run_variables step_info="runNumber $run_number, step_name $step_name, pipeline_name $pipeline_name"'
                    ]
                }
            },
            id: 3,
            name: 'add_run_variable',
            masterResourceId: 0,
            typeCode: 2007,
            latestStepId: 0,
            projectId: 1,
            pipelineSourceId: 1,
            pipelineId: 1,
            isDeleted: false,
            deletedAt: new Date('2023-06-21T12:09:18.904Z'),
            isConsistent: true,
            isStale: false,
            staleAt: new Date('2023-06-21T12:09:18.904Z'),
            isPaused: false,
            createdBy: '1',
            updatedBy: '1'
        }
    };
};
const mockPipeline = function () {
    return {
        ymlConfigPropertyBag: {},
        ymlPropertyBag: {
            name: 'intro_hello_world',
            steps: [
                {
                    name: 'first',
                    type: 'Bash',
                    execution: {
                        onExecute: ['jfrog --version']
                    }
                },
                {
                    name: 'variable_selection',
                    type: 'Bash',
                    configuration: {
                        inputSteps: [
                            {
                                name: 'first'
                            }
                        ],
                        environmentVariables: {
                            forceFail: {
                                default: 'false',
                                description: "set to 'true' to force the step to fail.",
                                values: ['false', 'true']
                            }
                        }
                    },
                    execution: {
                        onExecute: ['echo "This step will succeed"']
                    }
                }
            ]
        },
        id: 1,
        name: 'intro_hello_world',
        pipelineSourceBranch: '<pipelineSourceBranch>',
        projectId: 1,
        pipelineSourceId: 1,
        latestRunNumber: 0,
        latestRunId: 0,
        latestSuccessfulRunId: 0,
        latestFailedRunId: 0,
        latestErrorRunId: 0,
        latestSkippedRunId: 0,
        latestTimedOutRunId: 0,
        latestCancelledRunId: 0,
        latestUnstableRunId: 0,
        latestCompletedRunId: 0,
        syntaxVersion: 'v1.0',
        isDeleted: false,
        deletedAt: new Date('2023-06-21T12:09:18.826Z')
    };
};
const NUMBER_FIFTEEN = '15';
describe('createStepJson', () => {
    const jFrogPipelinesSessionId = '<jFrogPipelinesSessionId>';
    it('throws error for empty step', async () => {
        const createStepJson = new createStepJson_1.CreateStepJson({
            jFrogPipelinesSessionId
        });
        await expect(createStepJson.createPipeInfo()).rejects.toThrow('Step is empty');
    });
    it('pass for request with empty projectIntegrationsByName', async () => {
        const createStepJson = new createStepJson_1.CreateStepJson({
            step: mockStep(),
            runResourceVersions: mockRunResourceVersions(),
            projectIntegrationsByName: {},
            triggeredByIdentity: mockTriggeredByIdentity(),
            runNumber: 1,
            pipelineSource: mockPipelineSource(),
            nodePool: mockNodePool(),
            masterIntegrationsById: {
                [NUMBER_FIFTEEN]: {
                    id: 15,
                    name: 'githubEnterprise',
                    type: 'scm',
                    typeCode: 5000,
                    displayName: 'Github Enterprise',
                    isEnabled: true
                }
            },
            masterResourcesById: {},
            pipeline: mockPipeline(),
            pipelineStepsByName: mockPipelineStepsByName(),
            jFrogPipelinesSessionId
        });
        const ressult = await createStepJson.createPipeInfo();
        expect(ressult).toBeDefined();
    });
    it('pass for request with empty configPropertyBag', async () => {
        const mockedStep = mockStep();
        mockedStep.configPropertyBag = {};
        const createStepJson = new createStepJson_1.CreateStepJson({
            step: mockedStep,
            runResourceVersions: mockRunResourceVersions(),
            projectIntegrationsByName: {},
            triggeredByIdentity: mockTriggeredByIdentity(),
            runNumber: 1,
            pipelineSource: mockPipelineSource(),
            nodePool: mockNodePool(),
            masterIntegrationsById: {
                [NUMBER_FIFTEEN]: {
                    id: 15,
                    name: 'githubEnterprise',
                    type: 'scm',
                    typeCode: 5000,
                    displayName: 'Github Enterprise',
                    isEnabled: true
                }
            },
            masterResourcesById: {},
            pipeline: mockPipeline(),
            pipelineStepsByName: mockPipelineStepsByName(),
            jFrogPipelinesSessionId
        });
        const ressult = await createStepJson.createPipeInfo();
        expect(ressult).toBeDefined();
    });
    it('pass for request with empty integrations', async () => {
        const mockedStep = mockStep();
        mockedStep.configPropertyBag = {
            integrations: []
        };
        const createStepJson = new createStepJson_1.CreateStepJson({
            step: mockedStep,
            runResourceVersions: mockRunResourceVersions(),
            projectIntegrationsByName: {},
            triggeredByIdentity: mockTriggeredByIdentity(),
            runNumber: 1,
            pipelineSource: mockPipelineSource(),
            nodePool: mockNodePool(),
            masterIntegrationsById: {
                [NUMBER_FIFTEEN]: {
                    id: 15,
                    name: 'githubEnterprise',
                    type: 'scm',
                    typeCode: 5000,
                    displayName: 'Github Enterprise',
                    isEnabled: true
                }
            },
            masterResourcesById: {},
            pipeline: mockPipeline(),
            pipelineStepsByName: mockPipelineStepsByName(),
            jFrogPipelinesSessionId
        });
        const ressult = await createStepJson.createPipeInfo();
        expect(ressult).toBeDefined();
    });
    it('pass for request with empty input Steps', async () => {
        const mockedStep = mockStep();
        delete mockedStep.configPropertyBag.inputSteps;
        const createStepJson = new createStepJson_1.CreateStepJson({
            step: mockedStep,
            runResourceVersions: mockRunResourceVersions(),
            projectIntegrationsByName: {},
            triggeredByIdentity: mockTriggeredByIdentity(),
            runNumber: 1,
            pipelineSource: mockPipelineSource(),
            nodePool: mockNodePool(),
            masterIntegrationsById: {
                [NUMBER_FIFTEEN]: {
                    id: 15,
                    name: 'githubEnterprise',
                    type: 'scm',
                    typeCode: 5000,
                    displayName: 'Github Enterprise',
                    isEnabled: true
                }
            },
            masterResourcesById: {},
            pipeline: mockPipeline(),
            pipelineStepsByName: mockPipelineStepsByName(),
            jFrogPipelinesSessionId
        });
        const ressult = await createStepJson.createPipeInfo();
        expect(ressult).toBeDefined();
    });
    it('pass for valid request with projectIntegrationsByName', async () => {
        const createStepJson = new createStepJson_1.CreateStepJson({
            step: mockStep(),
            runResourceVersions: mockRunResourceVersions(),
            projectIntegrationsByName: {
                githubEnterprise: {
                    name: 'projectIntegrationsByName',
                    displayName: 'displayName'
                }
            },
            triggeredByIdentity: mockTriggeredByIdentity(),
            runNumber: 1,
            pipelineSource: mockPipelineSource(),
            nodePool: mockNodePool(),
            masterIntegrationsById: {
                [NUMBER_FIFTEEN]: {
                    id: 15,
                    name: 'githubEnterprise',
                    type: 'scm',
                    typeCode: 5000,
                    displayName: 'Github Enterprise',
                    isEnabled: true
                }
            },
            masterResourcesById: {},
            pipeline: mockPipeline(),
            pipelineStepsByName: mockPipelineStepsByName(),
            jFrogPipelinesSessionId
        });
        const ressult = await createStepJson.createPipeInfo();
        expect(ressult).toBeDefined();
    });
    it('pass for valid request with masterIntegrationId in projectIntegrationsByName', async () => {
        const createStepJson = new createStepJson_1.CreateStepJson({
            step: mockStep(),
            runResourceVersions: mockRunResourceVersions(),
            projectIntegrationsByName: {
                githubEnterprise: {
                    masterIntegrationId: NUMBER_FIFTEEN,
                    name: 'projectIntegrationsByName',
                    displayName: 'displayName'
                }
            },
            triggeredByIdentity: mockTriggeredByIdentity(),
            runNumber: 1,
            pipelineSource: mockPipelineSource(),
            nodePool: mockNodePool(),
            masterIntegrationsById: {
                [NUMBER_FIFTEEN]: {
                    id: 15,
                    name: 'githubEnterprise',
                    type: 'scm',
                    typeCode: 5000,
                    displayName: 'Github Enterprise',
                    isEnabled: true
                }
            },
            masterResourcesById: {},
            pipeline: mockPipeline(),
            pipelineStepsByName: mockPipelineStepsByName(),
            jFrogPipelinesSessionId
        });
        const ressult = await createStepJson.createPipeInfo();
        expect(ressult).toBeDefined();
    });
});
//# sourceMappingURL=createStepJson.test.js.map