"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateInfoJson = void 0;
const node_util_1 = require("node:util");
const errors_1 = require("@jfrog-pipelines/errors");
const utility_1 = require("@jfrog-pipelines/utility");
const SyncRepository_1 = require("../../acl/syncService/SyncRepository");
const constants_js_1 = require("../../constants.js");
const PIPELINE_OBJECTS_OBJECT_NAME = 'info';
const PIPELINE_OBJECTS_OBJECT_TYPE = 'run';
const PIPELINE_OBJECTS_STAGE = 'runTrigger';
class CreateInfoJson {
    constructor(payload) {
        this.run = payload.run;
        this.pipeline = payload.pipeline;
        this.pipelineSource = payload.pipelineSource;
        this.jFrogPipelinesSessionId = payload.jFrogPipelinesSessionId;
        this.syncRespository = new SyncRepository_1.SyncRepository();
    }
    async createPipeInfo() {
        try {
            this.validate();
            const pipelineObjectBody = {
                objectId: this.run.id,
                objectName: PIPELINE_OBJECTS_OBJECT_NAME,
                objectType: PIPELINE_OBJECTS_OBJECT_TYPE,
                stage: PIPELINE_OBJECTS_STAGE,
                data: this.createInfoJson()
            };
            const pipelineObject = await this.syncRespository.postPipelineObjects(pipelineObjectBody);
            return pipelineObject;
        }
        catch (error) {
            throw new errors_1.ApplicationError(`createPipeInfo for request ${this.jFrogPipelinesSessionId} failed with ${(0, node_util_1.inspect)(error)}`);
        }
    }
    validate() {
        if ((0, utility_1.isEmpty)(this.run)) {
            throw new errors_1.BadRequestError('Run is empty');
        }
        if ((0, utility_1.isEmpty)(this.pipeline)) {
            throw new errors_1.BadRequestError('Pipeline is empty');
        }
    }
    createInfoJson() {
        return {
            pipelineName: this.pipeline.name,
            branch: this.pipelineSource.isMultiBranch ? this.pipeline.pipelineSourceBranch : this.pipelineSource.branch,
            version: constants_js_1.PIPE_INFO_VERSION,
            runNumber: this.run.runNumber
        };
    }
}
exports.CreateInfoJson = CreateInfoJson;
//# sourceMappingURL=createInfoJson.js.map