"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiStepDataAdaptor = void 0;
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const metadata_1 = require("@jfrog-pipelines/metadata");
const configuration_1 = require("@jfrog-pipelines/configuration");
const SYSTEM_CODES_BY_NAME = (0, metadata_1.getSystemCodesByName)();
class ApiStepDataAdaptor {
    constructor() {
        this.properties = configuration_1.ConfigFactory.config;
        this.client = new internal_clients_1.PipelinesClient(16, this.properties.apiToken);
        this.batchSize = this.properties.defaultBatchSize;
    }
    async getStepByStepIds(stepIds) {
        if (stepIds.length === 0) {
            return [];
        }
        const query = `stepIds=${stepIds.join(',')}`;
        return this.client.getSteps(query);
    }
    async getWaitingStepsByPipelineIds(pipelineStepIds) {
        const waitingCode = SYSTEM_CODES_BY_NAME.waiting.code;
        const baseQuery = `pipelineStepIds=${pipelineStepIds.join(',')}&statusCodes=${waitingCode}&sortOrder=1&sortBy=id`;
        let skip = 0;
        let fetchedAllSteps = false;
        const stepList = [];
        while (!fetchedAllSteps) {
            const currentQuery = `${baseQuery}&limit=${this.batchSize}&skip=${skip}`;
            const steps = await this.client.getSteps(currentQuery);
            stepList.push(steps);
            skip += this.batchSize;
            fetchedAllSteps = steps.length !== this.batchSize;
        }
        return stepList.flat();
    }
}
exports.ApiStepDataAdaptor = ApiStepDataAdaptor;
//# sourceMappingURL=ApiStepDataAdaptor.js.map