"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiSyncDataAdaptor = void 0;
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const configuration_1 = require("@jfrog-pipelines/configuration");
class ApiSyncDataAdaptor {
    constructor() {
        this.properties = configuration_1.ConfigFactory.config;
        this.client = new internal_clients_1.PipelinesClient(16, this.properties.apiToken);
        this.batchSize = this.properties.defaultBatchSize;
    }
    async getPipelineSteps(ids) {
        const pipelineStepIds = Array.isArray(ids) ? ids : [ids];
        const query = `pipelineStepIds=${pipelineStepIds.join(',')}&include=id`;
        return this.client.getPipelineSteps(query);
    }
    async getResourceVersions(ids) {
        const queries = [];
        let currentIndex = 0;
        while (currentIndex < ids.length) {
            queries.push(ids.slice(currentIndex, currentIndex + this.batchSize));
            currentIndex += this.batchSize;
        }
        const promises = queries.map((query) => {
            return this.client.getResourceVersions(`resourceVersionIds=${query.join(',')}`);
        });
        const resourceVersions = await Promise.all(promises);
        return resourceVersions.flat();
    }
    async getResources(ids) {
        const queries = [];
        let currentIndex = 0;
        while (currentIndex < ids.length) {
            queries.push(ids.slice(currentIndex, currentIndex + this.batchSize));
            currentIndex += this.batchSize;
        }
        const promises = queries.map((query) => {
            return this.client.getResources(`resourceIds=${query.join(',')}`);
        });
        const result = await Promise.all(promises);
        return result.flat();
    }
    async getPipelineStepConnection(projectId, resourceIds) {
        const resourceIdGroups = [];
        let currentIndex = 0;
        while (currentIndex < resourceIds.length) {
            resourceIdGroups.push(resourceIds.slice(currentIndex, currentIndex + this.batchSize));
            currentIndex += this.batchSize;
        }
        const baseQuery = `projectIds=${projectId}&operationResourceIds=${resourceIdGroups.join(',')}&sortOrder=1&sortBy=id`;
        let skip = 0;
        let fetchedAllConnections = false;
        const connectionsList = [];
        while (!fetchedAllConnections) {
            const currentQuery = `${baseQuery}&limit=${this.batchSize}&skip=${skip}`;
            const connections = await this.client.pipelineStepConnections(currentQuery);
            connectionsList.push(connections);
            skip += this.batchSize;
            fetchedAllConnections = connections.length !== this.batchSize;
        }
        return connectionsList.flat();
    }
    async getPipelines(ids) {
        const pipelineIdGroup = [];
        let currentIndex = 0;
        while (currentIndex < ids.length) {
            pipelineIdGroup.push(ids.slice(currentIndex, currentIndex + this.batchSize));
            currentIndex += this.batchSize;
        }
        const query = `pipelineIds=${pipelineIdGroup.join(',')}&limit=${this.batchSize}`;
        return this.client.getPipelines(query);
    }
    async postPipelineObjects(pipelineObject) {
        return this.client.postPipelineObjects(pipelineObject);
    }
    async getTemplateById(id) {
        return this.client.getTemplateById(id);
    }
    async getPipelineById(id) {
        return this.client.getPipelineById(id);
    }
    getPipelineSource() {
        throw new Error('Method not implemented.');
    }
    getPipeline() {
        throw new Error('Method not implemented.');
    }
    getMasterResource() {
        throw new Error('Method not implemented.');
    }
}
exports.ApiSyncDataAdaptor = ApiSyncDataAdaptor;
//# sourceMappingURL=ApiSyncDataAdaptor.js.map