"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require('./ActErr.js');
const statusCodes = require('./statusCodes.js');
global.request = require('request');
const microservice_1 = require("@jfrog-pipelines/microservice");
const utility_1 = require("@jfrog-pipelines/utility");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const metadata_1 = require("@jfrog-pipelines/metadata");
const configuration_1 = require("@jfrog-pipelines/configuration");
const logger_1 = require("@jfrog-pipelines/logger");
const lodash_1 = __importDefault(require("lodash"));
const constants_1 = require("./constants");
const RunWorker_1 = require("./RunWorker");
const logLevel = 'info';
const QUEUE_NAME = 'core.runService';
const serviceEnum = 16;
const serviceDetails = utility_1.ServiceMeta[serviceEnum];
const options = {
    enum: serviceEnum,
    initializers: [new microservice_1.AmqpClientInitializer(), new microservice_1.ApiCheckInitializer(serviceEnum), new microservice_1.AccessClientInitializer()],
    healthChecks: [],
    models: [data_access_layer_1.Run, data_access_layer_1.RunAdjacency],
    logLevel: logLevel,
    validator: new microservice_1.DefaultApplicationValidator([], [])
};
async function startApp() {
    const app = new microservice_1.Application(options, microservice_1.applicationContext);
    const worker = new RunWorker_1.RunWorker(serviceDetails, microservice_1.applicationContext);
    global.util = require('node:util');
    global._ = require('underscore');
    global.async = require('async');
    global.config = {
        defaultPageSize: configuration_1.ConfigFactory.config.getCustom('core.defaultPageSize') ?? 100,
        defaultBatchSize: configuration_1.ConfigFactory.config.getCustom('core.defaultPageSize') ?? 300,
        jobConsoleBatchSize: 10,
        jobConsoleBufferTimeInterval: 10,
        languageImages: configuration_1.ConfigFactory.config.getCustom('runtime.languageImages') || [],
        autoSyncResourceIfOutdated: configuration_1.ConfigFactory.config.getCustom('core.autoSyncResourceIfOutdated') || false,
        signedPipelinesEnabled: configuration_1.ConfigFactory.config.getCustom('core.signedPipelinesEnabled') || false,
        apiToken: configuration_1.ConfigFactory.config.apiToken,
        rootBucket: configuration_1.ConfigFactory.config.getCustom('runtime.rootBucket'),
        instanceType: configuration_1.ConfigFactory.config.getCustom('runtime.instanceType') || 't3a.medium'
    };
    global.systemConfig = {
        stepTimeoutMS: configuration_1.ConfigFactory.config.stepTimeoutMS
    };
    global.systemCodesByName = (0, metadata_1.getSystemCodesByName)();
    global.statusCodes = statusCodes;
    lodash_1.default.each(global.systemCodesByGroup, (groupCodes, group) => {
        global.systemCodesByGroup[group] = lodash_1.default.keyBy(groupCodes, 'name');
    });
    global.systemCodesByCode = (0, metadata_1.getSystemCodesByCode)();
    global.systemCodes = (0, metadata_1.getSystemCodes)();
    global.logger = logger_1.LoggerFactory.getLogger('RunService');
    global.msName = 'runservice';
    global.request = global.request.defaults({
        headers: {
            Connection: 'keep-alive',
            'Keep-Alive': `timeout=${constants_1.KEEP_ALIVE.TIMEOUT_SEC}, max=${constants_1.KEEP_ALIVE.MAX_CONNECTIONS}`
        }
    });
    await app.listen(QUEUE_NAME, worker);
}
startApp();
//# sourceMappingURL=app.js.map