"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssemblerFactory = void 0;
const logger_1 = require("@jfrog-pipelines/logger");
const ChronologicalAssembler_1 = require("./chronological/ChronologicalAssembler");
const DependencyGraph_1 = require("./DependencyGraph");
const DependencyModeAssembler_1 = require("./dependencyMode/DependencyModeAssembler");
const InputStepAssembler_1 = require("./inputStep/InputStepAssembler");
class AssemblerFactory {
    static async getAssemblers(run, pipeline, steps, steplets, runResourceVersions, runStepConnections, dataLoader) {
        const logger = logger_1.LoggerFactory.getLogger(`AssemblerFactory:${run.id}`);
        logger.debug('Started');
        const inputStepAssembler = new InputStepAssembler_1.InputStepAssembler(steps, steplets, runResourceVersions, runStepConnections);
        const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler(pipeline, dataLoader);
        const chronologicalAssembler = new ChronologicalAssembler_1.ChronologicalAssembler(steps, pipeline, run, dataLoader);
        const [inputStep, dependencyMode, chronological] = await Promise.all([
            inputStepAssembler.populate(),
            dependencyModeAssembler.populate(),
            chronologicalAssembler.populate()
        ]);
        const dependencyGraph = new DependencyGraph_1.DependencyGraph(inputStep);
        const dependencyNodes = dependencyGraph.populate();
        logger.debug('Completed');
        return [inputStep, dependencyMode, chronological, dependencyNodes];
    }
}
exports.AssemblerFactory = AssemblerFactory;
//# sourceMappingURL=AssemblerFactory.js.map