"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunAdjacencyComputerChain = void 0;
const logger_1 = require("@jfrog-pipelines/logger");
const ChronologicalComputerChain_1 = require("./chronological/ChronologicalComputerChain");
const DependencyModeHandlerChain_1 = require("./dependencyMode/DependencyModeHandlerChain");
const ApprovalGatesHandler_1 = require("./inputStep/ApprovalGatesHandler");
const InputStepHandler_1 = require("./inputStep/InputStepHandler");
const RunAdjacencyContext_1 = require("./RunAdjacencyContext");
const Utils_1 = require("./Utils");
class RunAdjacencyComputerChain {
    constructor(dependendencyNodes, step, chronological, dependencyMode) {
        this.dependendencyNodes = dependendencyNodes;
        this.step = step;
        this.chronological = chronological;
        this.dependencyMode = dependencyMode;
        this.logger = logger_1.LoggerFactory.getLogger('RunAdjacencyComputerChain');
    }
    compute() {
        this.logger.debug('Started');
        const firstStep = this.step.getSteps()[0];
        const baseRunAdjacency = {
            projectId: firstStep.projectId,
            pipelineId: firstStep.pipelineId,
            runId: firstStep.runId,
            pipelineSourceId: firstStep.pipelineSourceId
        };
        const inputStepMap = (0, Utils_1.getInputStepMap)(this.dependendencyNodes);
        this.logger.trace(`Input step map for ${firstStep.runId}`, JSON.stringify(inputStepMap, null, 1));
        const firstSteps = this.step.getFirstSteps(inputStepMap);
        const context = new RunAdjacencyContext_1.RunAdjacencyContext(baseRunAdjacency);
        const inputStepHandler = new InputStepHandler_1.InputStepHandler(this.step, inputStepMap);
        const approvalGatesHandler = new ApprovalGatesHandler_1.ApprovalGatesHandler(this.step);
        const chronological = new ChronologicalComputerChain_1.ChronologicalComputerChain(this.step, this.chronological, firstSteps);
        const dependencyModeHandler = new DependencyModeHandlerChain_1.DependencyModeHandlerChain(this.step, this.dependencyMode, firstSteps);
        inputStepHandler.setNext(approvalGatesHandler).setNext(chronological).setNext(dependencyModeHandler);
        inputStepHandler.handle(context);
        const records = context.getValue();
        this.logger.info(`Computed ${records.length} records for ${firstStep.runId}`);
        return records;
    }
}
exports.RunAdjacencyComputerChain = RunAdjacencyComputerChain;
//# sourceMappingURL=RunAdjacencyComputerChain.js.map