"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const DependencyGraph_1 = require("../DependencyGraph");
const InputStep_1 = require("../inputStep/InputStep");
const Utils_1 = require("../Utils");
const resourceTest_json_1 = __importDefault(require("./mockData/resourceTest.json"));
const complexPipeline_json_1 = __importDefault(require("./mockData/complexPipeline.json"));
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
describe('DependencyGraph test cases', () => {
    it('should be defined', () => {
        const inputStep = new InputStep_1.InputStep([], [], [], []);
        const dependencyGraph = new DependencyGraph_1.DependencyGraph(inputStep);
        expect(dependencyGraph).toBeDefined();
    });
});
describe('DependencyGraph populate cases', () => {
    it('should get dependencyNodes', () => {
        const { steps, steplets, runStepConnections, runResourceVersions } = resourceTest_json_1.default;
        const inputStep = new InputStep_1.InputStep(steps, steplets, runResourceVersions, runStepConnections);
        const dependencyGraph = new DependencyGraph_1.DependencyGraph(inputStep);
        const result = dependencyGraph.populate();
        const inputStepMap = (0, Utils_1.getInputStepMap)(result);
        expect(inputStepMap).toStrictEqual({
            '2417': [2419],
            '2418': [2420],
            '2419': [],
            '2420': [2417, 2421],
            '2421': [2419]
        });
        expect(inputStepMap).toBeDefined();
    });
    it('should get verify inputStepMap', () => {
        const { steps, steplets, runStepConnections, runResourceVersions } = complexPipeline_json_1.default;
        const inputStep = new InputStep_1.InputStep(steps, steplets, runResourceVersions, runStepConnections);
        const dependencyGraph = new DependencyGraph_1.DependencyGraph(inputStep);
        const result = dependencyGraph.populate();
        const inputStepMap = (0, Utils_1.getInputStepMap)(result);
        expect(inputStepMap).toStrictEqual({
            '4415': [],
            '4416': [4419],
            '4417': [4427, 4494],
            '4418': [],
            '4419': [4425, 4417],
            '4420': [],
            '4421': [],
            '4422': [],
            '4423': [4448, 4469],
            '4424': [],
            '4425': [4429],
            '4426': [4430, 4429],
            '4427': [4442],
            '4428': [],
            '4429': [4484],
            '4430': [4509],
            '4431': [],
            '4432': [4426],
            '4433': [4494],
            '4434': [4433],
            '4435': [],
            '4436': [],
            '4437': [],
            '4438': [4419],
            '4439': [],
            '4440': [4504],
            '4441': [],
            '4442': [4429, 4484],
            '4443': [],
            '4444': [4448],
            '4445': [],
            '4446': [],
            '4447': [],
            '4448': [4494],
            '4449': [4484],
            '4450': [],
            '4451': [],
            '4452': [4430],
            '4453': [4499],
            '4454': [],
            '4455': [],
            '4456': [4429, 4417, 4430],
            '4457': [],
            '4458': [],
            '4459': [],
            '4460': [],
            '4461': [],
            '4462': [4502],
            '4463': [],
            '4464': [],
            '4465': [],
            '4466': [],
            '4467': [],
            '4468': [],
            '4469': [4448],
            '4470': [],
            '4471': [],
            '4472': [],
            '4473': [],
            '4474': [],
            '4475': [],
            '4476': [],
            '4477': [4419, 4434],
            '4478': [],
            '4479': [],
            '4480': [],
            '4481': [],
            '4482': [],
            '4483': [],
            '4484': [4496],
            '4485': [],
            '4486': [],
            '4487': [],
            '4488': [],
            '4489': [],
            '4490': [],
            '4491': [],
            '4492': [],
            '4493': [4495],
            '4494': [],
            '4495': [4426],
            '4496': [],
            '4497': [4433],
            '4498': [],
            '4499': [4503, 4438],
            '4500': [],
            '4501': [],
            '4502': [4419],
            '4503': [4445],
            '4504': [4509, 4429],
            '4505': [],
            '4506': [],
            '4507': [],
            '4508': [],
            '4509': [],
            '4510': [],
            '4511': [],
            '4512': [],
            '4513': [4417],
            '4514': [],
            '4515': []
        });
    });
});
//# sourceMappingURL=DependencyGraph.test.js.map