"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChronologicalRunHandler = void 0;
const logger_1 = require("@jfrog-pipelines/logger");
const Utils_1 = require("../Utils");
const AbstractHandler_1 = require("../chain/AbstractHandler");
class ChronologicalRunHandler extends AbstractHandler_1.AbstractHandler {
    constructor(inputStep, chronological, firstSteps) {
        super();
        this.inputStep = inputStep;
        this.chronological = chronological;
        this.firstSteps = firstSteps;
        this.logger = logger_1.LoggerFactory.getLogger('ChronologicalRunHandler');
    }
    handle(context) {
        this.logger.debug('Started');
        const previousRun = this.chronological.getPreviousRun();
        if (!this.chronological.hasPipelineFlagEnabled() || !previousRun) {
            this.logger.trace('Chronological flag not enabled. Hence returning');
            super.handle(context);
            return;
        }
        for (const step of this.firstSteps) {
            const computedRecords = this.compute(context.getBaseRunAdjacency(), step, previousRun);
            this.logger.trace(`Computed records for step ${step.id}: ${computedRecords.length}`);
            context.addRecords(computedRecords);
        }
        super.handle(context);
    }
    compute(baseRunAdjacencyObject, step, previousRun) {
        const records = [];
        const affinityGroupSteps = this.inputStep.getStepsByAffinityGroup(step.affinityGroup);
        for (const step of affinityGroupSteps) {
            const steplets = this.inputStep.getStepletsByStepId(step.id);
            records.push(...(0, Utils_1.createRunAdjacencyRecords)(baseRunAdjacencyObject, step, steplets, {
                dependentEntityId: previousRun.id,
                dependentEntityType: 2
            }));
        }
        return records;
    }
}
exports.ChronologicalRunHandler = ChronologicalRunHandler;
//# sourceMappingURL=ChronologicalRunHandler.js.map