"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChronologicalStepHandler = void 0;
const logger_1 = require("@jfrog-pipelines/logger");
const Utils_1 = require("../Utils");
const AbstractHandler_1 = require("../chain/AbstractHandler");
class ChronologicalStepHandler extends AbstractHandler_1.AbstractHandler {
    constructor(step, chronological) {
        super();
        this.step = step;
        this.chronological = chronological;
        this.logger = logger_1.LoggerFactory.getLogger('ChronologicalStepHandler');
    }
    handle(context) {
        this.logger.debug('Started');
        const chronologicalSteps = this.chronological.getPreviousSteps();
        this.logger.trace('Chronological steps', chronologicalSteps.length);
        for (const { step, previousStep } of chronologicalSteps) {
            const computedRecords = this.compute(context.getBaseRunAdjacency(), step, previousStep);
            this.logger.trace(`computed records for ${step.id}: ${computedRecords.length}`);
            context.addRecords(computedRecords);
        }
        super.handle(context);
    }
    compute(baseRunAdjacencyObject, step, previousStep) {
        const records = [];
        const affinityGroupSteps = this.step.getStepsByAffinityGroup(step.affinityGroup);
        for (const affinityGroupStep of affinityGroupSteps) {
            const steplets = this.step.getStepletsByStepId(affinityGroupStep.id);
            records.push(...(0, Utils_1.createRunAdjacencyRecords)(baseRunAdjacencyObject, step, steplets, {
                dependentEntityId: previousStep.id,
                dependentEntityType: 1
            }));
        }
        return records;
    }
}
exports.ChronologicalStepHandler = ChronologicalStepHandler;
//# sourceMappingURL=ChronologicalStepHandler.js.map