"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const ApiDataLoader_1 = require("../../dataLoader/ApiDataLoader");
const ChronologicalAssembler_1 = require("../ChronologicalAssembler");
class MockApiDataLoader extends ApiDataLoader_1.ApiDataLoader {
}
jest.mock('@jfrog-pipelines/configuration', () => ({
    ConfigFactory: {
        config: {
            apiUrl: 'http://example.com',
            controlPlaneVersion: '1.23.6'
        }
    }
}));
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const pipelinesClient = new internal_clients_1.PipelinesClient(0, 'token');
const mockApiDataLoader = new MockApiDataLoader(pipelinesClient);
describe('ChronologicalAssembler init', () => {
    const chronologicalAssembler = new ChronologicalAssembler_1.ChronologicalAssembler([], {}, {}, new ApiDataLoader_1.ApiDataLoader(new internal_clients_1.PipelinesClient(5, 'token')));
    it('should be defined', () => {
        expect(chronologicalAssembler).toBeDefined();
    });
});
describe('ChronologicalAssembler populate', () => {
    it('should populate chronologicalSteps empty', async () => {
        const chronologicalAssembler = new ChronologicalAssembler_1.ChronologicalAssembler([], {}, {}, mockApiDataLoader);
        const chronlogical = await chronologicalAssembler.populate();
        expect(chronlogical).toBeDefined();
    });
    it('should populate chronologicalSteps not empty', async () => {
        const chronologicalAssembler = new ChronologicalAssembler_1.ChronologicalAssembler([
            {
                runId: 3,
                id: 3,
                pipelineStepId: 1,
                configPropertyBag: {
                    chronological: true
                }
            }
        ], {}, {
            id: 3
        }, mockApiDataLoader);
        const getChronologicalStepsSpy = jest.spyOn(mockApiDataLoader, 'getChronologicalSteps').mockResolvedValueOnce([
            {
                runId: 1,
                id: 1,
                pipelineStepId: 1
            },
            {
                runId: 2,
                id: 2,
                pipelineStepId: 1
            },
            {
                runId: 3,
                id: 3,
                pipelineStepId: 1
            },
            {
                runId: 4,
                id: 4,
                pipelineStepId: 1
            }
        ]);
        const chronlogical = await chronologicalAssembler.populate();
        expect(chronlogical).toBeDefined();
        expect(chronologicalAssembler).toBeDefined();
        expect(getChronologicalStepsSpy).toHaveBeenCalledTimes(1);
        const previousSteps = chronlogical.getPreviousSteps();
        expect(previousSteps).toHaveLength(1);
        const { step, previousStep } = previousSteps[0];
        expect(step).toStrictEqual({
            runId: 3,
            id: 3,
            pipelineStepId: 1,
            configPropertyBag: {
                chronological: true
            }
        });
        expect(previousStep).toStrictEqual({
            runId: 2,
            id: 2,
            pipelineStepId: 1
        });
    });
    it('should populate chronologicalSteps is empty', async () => {
        const chronologicalAssembler = new ChronologicalAssembler_1.ChronologicalAssembler([
            {
                runId: 3,
                id: 3,
                pipelineStepId: 1,
                configPropertyBag: {
                    chronological: true
                }
            }
        ], {}, {
            id: 3
        }, mockApiDataLoader);
        const getChronologicalStepsSpy = jest
            .spyOn(mockApiDataLoader, 'getChronologicalSteps')
            .mockResolvedValueOnce([]);
        const chronlogical = await chronologicalAssembler.populate();
        expect(chronlogical).toBeDefined();
        expect(chronologicalAssembler).toBeDefined();
        expect(getChronologicalStepsSpy).toHaveBeenCalledTimes(1);
        const previousSteps = chronlogical.getPreviousSteps();
        expect(previousSteps).toHaveLength(0);
    });
    it('should populate chronologicalRun not empty', async () => {
        const chronologicalAssembler = new ChronologicalAssembler_1.ChronologicalAssembler([
            {
                runId: 3,
                id: 1,
                pipelineStepId: 1
            },
            {
                runId: 3,
                id: 2,
                pipelineStepId: 1
            },
            {
                runId: 3,
                id: 3,
                pipelineStepId: 1
            }
        ], {
            ymlConfigPropertyBag: {
                chronological: true
            }
        }, {
            id: 3
        }, mockApiDataLoader);
        const getPreviousRunSpy = jest.spyOn(mockApiDataLoader, 'getRuns').mockResolvedValueOnce([
            {
                id: 1,
                runNumber: 1
            },
            {
                id: 2,
                runNumber: 2
            },
            {
                id: 3,
                runNumber: 3
            }
        ]);
        const chronlogical = await chronologicalAssembler.populate();
        expect(chronlogical).toBeDefined();
        expect(getPreviousRunSpy).toHaveBeenCalledTimes(1);
        const previousRun = chronlogical.getPreviousRun();
        expect(previousRun).toStrictEqual({
            id: 2,
            runNumber: 2
        });
    });
    it('should populate chronologicalRun is empty', async () => {
        const chronologicalAssembler = new ChronologicalAssembler_1.ChronologicalAssembler([
            {
                runId: 3,
                id: 1,
                pipelineStepId: 1
            }
        ], {
            ymlConfigPropertyBag: {
                chronological: true
            }
        }, {
            id: 3
        }, mockApiDataLoader);
        const getPreviousRunSpy = jest.spyOn(mockApiDataLoader, 'getRuns').mockResolvedValueOnce([
            {
                id: 3,
                runNumber: 3
            }
        ]);
        const chronlogical = await chronologicalAssembler.populate();
        expect(chronlogical).toBeDefined();
        expect(getPreviousRunSpy).toHaveBeenCalledTimes(1);
        const previousRun = chronlogical.getPreviousRun();
        expect(previousRun).toStrictEqual(null);
    });
});
//# sourceMappingURL=chronologicalAssembler.test.js.map