"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const InputStep_1 = require("../../inputStep/InputStep");
const RunAdjacencyContext_1 = require("../../RunAdjacencyContext");
const Chronological_1 = require("../Chronological");
const ChronologicalRunHandler_1 = require("../ChronologicalRunHandler");
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
describe('ChronologicalRunHandler init', () => {
    it('should be defined', () => {
        const chronlogicalRunHandler = new ChronologicalRunHandler_1.ChronologicalRunHandler({}, {}, []);
        expect(chronlogicalRunHandler).toBeDefined();
    });
});
describe('ChronologicalRunHandler handle', () => {
    const steps = [
        {
            id: 1,
            affinityGroup: 'one'
        },
        {
            id: 2,
            affinityGroup: 'two'
        }
    ];
    const steplets = [
        {
            id: 1,
            stepId: 1,
            affinityGroup: 'one'
        },
        {
            id: 2,
            stepId: 2,
            affinityGroup: 'two'
        }
    ];
    it('should handle when flag enabled', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const firstSteps = [
            {
                id: 1,
                affinityGroup: 'one'
            },
            {
                id: 2,
                affinityGroup: 'two'
            }
        ];
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const chronlogical = new Chronological_1.Chronological([], {});
        chronlogical.setPreviousRun({ id: 1, runNumber: 1 });
        chronlogical.setPipelineFlagEnabled(true);
        const chronlogicalRunHandler = new ChronologicalRunHandler_1.ChronologicalRunHandler(inputStep, chronlogical, firstSteps);
        expect(chronlogicalRunHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        chronlogicalRunHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(2);
        expect(context.getValue()).toStrictEqual([
            {
                runId: 1,
                dependentEntityId: 1,
                dependentEntityType: 2,
                affinityGroup: 'one',
                stepId: 1,
                stepletId: 1
            },
            {
                runId: 1,
                dependentEntityId: 1,
                dependentEntityType: 2,
                affinityGroup: 'two',
                stepId: 2,
                stepletId: 2
            }
        ]);
    });
    it('should handle when flag disabled', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const firstSteps = [
            {
                id: 1,
                affinityGroup: 'one'
            },
            {
                id: 2,
                affinityGroup: 'two'
            }
        ];
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const chronlogical = new Chronological_1.Chronological([], {});
        chronlogical.setPipelineFlagEnabled(false);
        const chronlogicalRunHandler = new ChronologicalRunHandler_1.ChronologicalRunHandler(inputStep, chronlogical, firstSteps);
        expect(chronlogicalRunHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        chronlogicalRunHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(0);
    });
    it('should handle when flag enabled with no previous run', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const firstSteps = [
            {
                id: 1,
                affinityGroup: 'one'
            },
            {
                id: 2,
                affinityGroup: 'two'
            }
        ];
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const chronlogical = new Chronological_1.Chronological([], {});
        chronlogical.setPipelineFlagEnabled(true);
        const chronlogicalRunHandler = new ChronologicalRunHandler_1.ChronologicalRunHandler(inputStep, chronlogical, firstSteps);
        expect(chronlogicalRunHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        chronlogicalRunHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(0);
    });
});
//# sourceMappingURL=chronologicalRunHandler.test.js.map