"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const InputStep_1 = require("../../inputStep/InputStep");
const RunAdjacencyContext_1 = require("../../RunAdjacencyContext");
const Chronological_1 = require("../Chronological");
const ChronologicalStepHandler_1 = require("../ChronologicalStepHandler");
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
describe('ChronologicalStepHandler init', () => {
    it('should be defined', () => {
        const chronlogicalStepHandler = new ChronologicalStepHandler_1.ChronologicalStepHandler({}, {});
        expect(chronlogicalStepHandler).toBeDefined();
    });
});
describe('ChronologicalStepHandler handle', () => {
    const steps = [
        {
            id: 2,
            affinityGroup: 'two'
        }
    ];
    const steplets = [
        {
            id: 2,
            stepId: 2,
            affinityGroup: 'two'
        }
    ];
    it('should handle when there are previous steps', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const chronlogical = new Chronological_1.Chronological([], {});
        chronlogical.setPreviousSteps([
            {
                step: {
                    id: 2,
                    affinityGroup: 'two'
                },
                previousStep: {
                    id: 1,
                    affinityGroup: 'two'
                }
            }
        ]);
        const chronlogicalStepHandler = new ChronologicalStepHandler_1.ChronologicalStepHandler(inputStep, chronlogical);
        expect(chronlogicalStepHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        chronlogicalStepHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(1);
        expect(context.getValue()).toStrictEqual([
            {
                runId: 1,
                dependentEntityId: 1,
                dependentEntityType: 1,
                affinityGroup: 'two',
                stepId: 2,
                stepletId: 2
            }
        ]);
    });
    it('should handle when no previous steps', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const chronlogical = new Chronological_1.Chronological([], {});
        chronlogical.setPreviousSteps([]);
        const chronlogicalStepHandler = new ChronologicalStepHandler_1.ChronologicalStepHandler(inputStep, chronlogical);
        expect(chronlogicalStepHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        chronlogicalStepHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(0);
    });
});
//# sourceMappingURL=chronologicalStepHandler.test.js.map