"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiDataLoader = void 0;
const logger_1 = require("@jfrog-pipelines/logger");
const CALL_OBJECT_LIMIT = 100;
class ApiDataLoader {
    constructor(pipelinesClient) {
        this.pipelinesClient = pipelinesClient;
        this.logger = logger_1.LoggerFactory.getLogger('ApiDataLoader');
    }
    async getPipelines(query) {
        this.logger.debug('Fetching pipelines for query', query);
        let pipelines = [];
        let currentSkip = 0;
        let page = [];
        do {
            const currentQuery = `${query}&limit=${CALL_OBJECT_LIMIT}&skip=${currentSkip}`;
            page = await this.getPipelinesForQuey(currentQuery);
            pipelines = [...pipelines, ...page];
            currentSkip += CALL_OBJECT_LIMIT;
        } while (page.length === CALL_OBJECT_LIMIT);
        this.logger.debug(`Succesfully fetched ${pipelines.length} pipelines`);
        return pipelines;
    }
    async getPipelineStepConnections(query) {
        this.logger.debug('Fetching pipelineStepConnections for query', query);
        let pipelineStepConnections = [];
        let currentSkip = 0;
        let page = [];
        do {
            const currentQuery = `${query}&limit=${CALL_OBJECT_LIMIT}&skip=${currentSkip}`;
            page = await this.getPipelineStepConnectionsForQuey(currentQuery);
            pipelineStepConnections = [...pipelineStepConnections, ...page];
            currentSkip += CALL_OBJECT_LIMIT;
        } while (page.length === CALL_OBJECT_LIMIT);
        this.logger.debug(`Succesfully fetched ${pipelineStepConnections.length} pipelineStepConnections`);
        return pipelineStepConnections;
    }
    async getRuns(query) {
        this.logger.debug('Fetching runs for query', query);
        let runs = [];
        let currentSkip = 0;
        let page = [];
        do {
            const currentQuery = `${query}&limit=${CALL_OBJECT_LIMIT}&skip=${currentSkip}`;
            page = await this.getRunsForQuey(currentQuery);
            runs = [...runs, ...page];
            currentSkip += CALL_OBJECT_LIMIT;
        } while (page.length === CALL_OBJECT_LIMIT);
        this.logger.debug(`Succesfully fetched ${runs.length} runs`);
        return runs;
    }
    async getChronologicalSteps(query) {
        this.logger.debug('Fetching chronological steps for query', query);
        let steps = [];
        let currentSkip = 0;
        let page = [];
        do {
            const currentQuery = `${query}&limit=${CALL_OBJECT_LIMIT}&skip=${currentSkip}`;
            page = await this.getStepsForQuery(currentQuery);
            steps = [...steps, ...page];
            currentSkip += CALL_OBJECT_LIMIT;
        } while (page.length === CALL_OBJECT_LIMIT);
        this.logger.debug(`Succesfully fetched ${steps.length} steps`);
        return steps;
    }
    getPipelinesForQuey(query) {
        return this.pipelinesClient.getPipelines(query);
    }
    getPipelineStepConnectionsForQuey(query) {
        return this.pipelinesClient.pipelineStepConnections(query);
    }
    getRunsForQuey(query) {
        return this.pipelinesClient.getRuns(query);
    }
    getStepsForQuery(query) {
        return this.pipelinesClient.getSteps(query);
    }
}
exports.ApiDataLoader = ApiDataLoader;
//# sourceMappingURL=ApiDataLoader.js.map