"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const ApiDataLoader_1 = require("../ApiDataLoader");
jest.mock('@jfrog-pipelines/configuration', () => ({
    ConfigFactory: {
        config: {
            apiUrl: 'http://example.com',
            controlPlaneVersion: '1.23.6'
        }
    }
}));
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const pipelinesClient = new internal_clients_1.PipelinesClient(0, 'token');
describe('ApiDataLoader init', () => {
    it('should be defined', () => {
        const apiDataLoader = new ApiDataLoader_1.ApiDataLoader(pipelinesClient);
        expect(apiDataLoader).toBeDefined();
    });
});
describe('ApiDataLoader methods', () => {
    it('should get pipelines', async () => {
        const apiDataLoader = new ApiDataLoader_1.ApiDataLoader(pipelinesClient);
        const getPipelinesSpy = jest.spyOn(pipelinesClient, 'getPipelines').mockResolvedValue([]);
        const pipelines = await apiDataLoader.getPipelines('pipelineIds=1');
        expect(getPipelinesSpy).toHaveBeenCalledTimes(1);
        expect(pipelines).toStrictEqual([]);
    });
    it('should get runs', async () => {
        const apiDataLoader = new ApiDataLoader_1.ApiDataLoader(pipelinesClient);
        const spy = jest.spyOn(pipelinesClient, 'getRuns').mockResolvedValue([]);
        const runs = await apiDataLoader.getRuns('pipelineIds=1');
        expect(spy).toHaveBeenCalledTimes(1);
        expect(runs).toStrictEqual([]);
    });
    it('should get chronologicalSteps', async () => {
        const apiDataLoader = new ApiDataLoader_1.ApiDataLoader(pipelinesClient);
        const chronologicalStepsSpy = jest.spyOn(pipelinesClient, 'getSteps').mockResolvedValue([]);
        const chronlogicalSteps = await apiDataLoader.getChronologicalSteps('pipelineIds=1');
        expect(chronologicalStepsSpy).toHaveBeenCalledTimes(1);
        expect(chronlogicalSteps).toStrictEqual([]);
    });
    it('should get pipelineStepConnections', async () => {
        const apiDataLoader = new ApiDataLoader_1.ApiDataLoader(pipelinesClient);
        const pipelineStepConnectionsSpy = jest.spyOn(pipelinesClient, 'pipelineStepConnections').mockResolvedValue([]);
        const pipelineStepConnections = await apiDataLoader.getPipelineStepConnections('pipelineIds=1');
        expect(pipelineStepConnectionsSpy).toHaveBeenCalledTimes(1);
        expect(pipelineStepConnections).toStrictEqual([]);
    });
});
//# sourceMappingURL=ApiDataLoader.test.js.map