"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WaitOnParentSuccessDependencyModeHandler = void 0;
const AbstractHandler_1 = require("../chain/AbstractHandler");
const Utils_1 = require("../Utils");
class WaitOnParentSuccessDependencyModeHandler extends AbstractHandler_1.AbstractHandler {
    constructor(step, dependencyMode, firstSteps) {
        super();
        this.step = step;
        this.dependencyMode = dependencyMode;
        this.firstSteps = firstSteps;
    }
    handle(context) {
        const dependencyModeSettings = this.dependencyMode.getDependencyModeSettings();
        if (!dependencyModeSettings.waitOnParentSuccess) {
            super.handle(context);
            return;
        }
        const { waitOnParentSuccessPipelineIds } = this.dependencyMode.getDependencyModePipelines();
        for (const pipelineId of waitOnParentSuccessPipelineIds) {
            for (const step of this.firstSteps) {
                const computedRecords = this.compute(context.getBaseRunAdjacency(), step, pipelineId);
                context.addRecords(computedRecords);
            }
        }
        super.handle(context);
    }
    compute(baseRunAdjacencyObject, step, pipelineId) {
        const records = [];
        const affinityGroupSteps = this.step.getStepsByAffinityGroup(step.affinityGroup);
        for (const affinityGroupStep of affinityGroupSteps) {
            const steplets = this.step.getStepletsByStepId(affinityGroupStep.id);
            records.push(...(0, Utils_1.createRunAdjacencyRecords)(baseRunAdjacencyObject, step, steplets, {
                dependentEntityId: pipelineId,
                dependentEntityType: 4
            }));
        }
        return records;
    }
}
exports.WaitOnParentSuccessDependencyModeHandler = WaitOnParentSuccessDependencyModeHandler;
//# sourceMappingURL=WaitOnParentSuccessDependencyModeHandler.js.map