"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const ApiDataLoader_1 = require("../../dataLoader/ApiDataLoader");
const DependencyModeAssembler_1 = require("../DependencyModeAssembler");
class MockApiDataLoader extends ApiDataLoader_1.ApiDataLoader {
}
jest.mock('@jfrog-pipelines/configuration', () => ({
    ConfigFactory: {
        config: {
            apiUrl: 'http://example.com',
            controlPlaneVersion: '1.23.6'
        }
    }
}));
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const pipelinesClient = new internal_clients_1.PipelinesClient(0, 'token');
const mockApiDataLoader = new MockApiDataLoader(pipelinesClient);
describe('DependencyModeAssembler init', () => {
    const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler({}, mockApiDataLoader);
    it('should be defined', async () => {
        expect(dependencyModeAssembler).toBeDefined();
        await dependencyModeAssembler.populate();
    });
});
describe('DependencyModeAssembler populate', () => {
    it('should populate when dependencyMode is disabled', async () => {
        const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler({ id: 1, ymlConfigPropertyBag: {} }, mockApiDataLoader);
        const dependencyMode = await dependencyModeAssembler.populate();
        expect(dependencyMode.isDependencyModeEnabled()).toStrictEqual(false);
    });
    it('should populate when waitOnParentComplete is enabled', async () => {
        const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler({
            id: 2,
            ymlConfigPropertyBag: {
                dependencyMode: {
                    waitOnParentComplete: true
                }
            }
        }, mockApiDataLoader);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'IN',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 2
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'OUT',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 1
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getRuns').mockResolvedValueOnce([
            {
                id: 1,
                pipelineId: 1,
                statusCode: 4001
            }
        ]);
        const dependencyMode = await dependencyModeAssembler.populate();
        const dependencyModePipelines = dependencyMode.getDependencyModePipelines();
        expect(dependencyModePipelines.waitOnParentCompletePipelineIds).toStrictEqual([1]);
        expect(dependencyMode.isDependencyModeEnabled()).toStrictEqual(true);
    });
    it('should populate when waitOnParentSuccess is enabled with processing parent', async () => {
        const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler({
            id: 2,
            ymlConfigPropertyBag: {
                dependencyMode: {
                    waitOnParentSuccess: true
                }
            }
        }, mockApiDataLoader);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'IN',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 2
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'OUT',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 1
            }
        ]);
        const dependencyMode = await dependencyModeAssembler.populate();
        const dependencyModePipelines = dependencyMode.getDependencyModePipelines();
        expect(dependencyModePipelines.waitOnParentSuccessPipelineIds).toStrictEqual([1]);
        expect(dependencyMode.isDependencyModeEnabled()).toStrictEqual(true);
    });
    it('should populate when waitOnParentSuccess is enabled with successful parent', async () => {
        const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler({
            id: 2,
            ymlConfigPropertyBag: {
                dependencyMode: {
                    waitOnParentSuccess: true
                }
            }
        }, mockApiDataLoader);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'IN',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 2
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'OUT',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 1
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getPipelines').mockResolvedValueOnce([
            {
                id: 1,
                latestCompletedRunId: 10,
                latestSuccessfulRunId: 10
            }
        ]);
        const dependencyMode = await dependencyModeAssembler.populate();
        const dependencyModePipelines = dependencyMode.getDependencyModePipelines();
        expect(dependencyModePipelines.waitOnParentSuccessPipelineIds).toStrictEqual([1]);
        expect(dependencyMode.isDependencyModeEnabled()).toStrictEqual(true);
    });
    it('should populate when waitOnChildComplete is enabled', async () => {
        const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler({
            id: 1,
            ymlConfigPropertyBag: {
                dependencyMode: {
                    waitOnChildComplete: true
                }
            }
        }, mockApiDataLoader);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'OUT',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 1
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'IN',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 2
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getRuns').mockResolvedValueOnce([
            {
                id: 1,
                pipelineId: 2,
                statusCode: 4001
            }
        ]);
        const dependencyMode = await dependencyModeAssembler.populate();
        const dependencyModePipelines = dependencyMode.getDependencyModePipelines();
        expect(dependencyModePipelines.waitOnChildCompletePipelineIds).toStrictEqual([2]);
        expect(dependencyMode.isDependencyModeEnabled()).toStrictEqual(true);
    });
    it('should populate when waitOnChildComplete is enabled and has completed runs', async () => {
        const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler({
            id: 1,
            ymlConfigPropertyBag: {
                dependencyMode: {
                    waitOnChildComplete: true
                }
            }
        }, mockApiDataLoader);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'OUT',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 1
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'IN',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 2
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getRuns').mockResolvedValueOnce([]);
        const dependencyMode = await dependencyModeAssembler.populate();
        const dependencyModePipelines = dependencyMode.getDependencyModePipelines();
        expect(dependencyModePipelines.waitOnChildCompletePipelineIds).toStrictEqual([2]);
        expect(dependencyMode.isDependencyModeEnabled()).toStrictEqual(true);
    });
    it('should populate when waitOnChildComplete, waitOnParentComplete and waitOnParentSuccess is enabled and has completed runs', async () => {
        const childPipelineId = 10;
        const parentSuccessPipelineId = 11;
        const parentCompletePipelineId = 12;
        const dependencyModeAssembler = new DependencyModeAssembler_1.DependencyModeAssembler({
            id: 1,
            ymlConfigPropertyBag: {
                dependencyMode: {
                    waitOnChildComplete: true,
                    waitOnParentComplete: true,
                    waitOnParentSuccess: true
                }
            }
        }, mockApiDataLoader);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'OUT',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: 1
            },
            {
                operation: 'IN',
                operationResourceId: 2,
                isTrigger: true,
                pipelineId: 1
            },
            {
                operation: 'IN',
                operationResourceId: 3,
                isTrigger: true,
                pipelineId: 1
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getPipelineStepConnections').mockResolvedValueOnce([
            {
                operation: 'IN',
                operationResourceId: 1,
                isTrigger: true,
                pipelineId: childPipelineId
            },
            {
                operation: 'OUT',
                operationResourceId: 2,
                isTrigger: true,
                pipelineId: parentSuccessPipelineId
            },
            {
                operation: 'OUT',
                operationResourceId: 3,
                isTrigger: true,
                pipelineId: parentCompletePipelineId
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getRuns').mockResolvedValueOnce([
            {
                id: 1,
                pipelineId: childPipelineId,
                statusCode: 4001
            },
            {
                id: 2,
                pipelineId: parentCompletePipelineId,
                statusCode: 4001
            }
        ]);
        jest.spyOn(mockApiDataLoader, 'getRuns').mockResolvedValueOnce([]);
        jest.spyOn(mockApiDataLoader, 'getPipelines').mockResolvedValueOnce([
            {
                id: parentSuccessPipelineId,
                latestCompletedRunId: 11,
                latestSuccessfulRunId: 12
            }
        ]);
        const dependencyMode = await dependencyModeAssembler.populate();
        const dependencyModePipelines = dependencyMode.getDependencyModePipelines();
        expect(dependencyModePipelines.waitOnChildCompletePipelineIds).toStrictEqual([10]);
        expect(dependencyModePipelines.waitOnParentCompletePipelineIds).toStrictEqual([11, 12]);
        expect(dependencyModePipelines.waitOnParentSuccessPipelineIds).toStrictEqual([11, 12]);
        expect(dependencyMode.isDependencyModeEnabled()).toStrictEqual(true);
    });
});
//# sourceMappingURL=dependencyModeAssembler.test.js.map