"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const RunAdjacencyContext_1 = require("../../RunAdjacencyContext");
const DependencyMode_1 = require("../DependencyMode");
const DependencyModeHandlerChain_1 = require("../DependencyModeHandlerChain");
const WaitOnChildCompleteDependencyModeHandler_1 = require("../WaitOnChildCompleteDependencyModeHandler");
const WaitOnParentCompleteDependencyModeHandler_1 = require("../WaitOnParentCompleteDependencyModeHandler");
const WaitOnParentSuccessDependencyModeHandler_1 = require("../WaitOnParentSuccessDependencyModeHandler");
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
describe('DependencyModeComputerChain int', () => {
    it('should be defined', () => {
        const dependencyModeHandlerChain = new DependencyModeHandlerChain_1.DependencyModeHandlerChain({}, {}, []);
        expect(dependencyModeHandlerChain).toBeDefined();
    });
    it('should call handle with context', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const dependencyMode = new DependencyMode_1.DependencyMode();
        dependencyMode.setDependencyModePipelines({
            waitOnChildCompletePipelineIds: [],
            waitOnParentCompletePipelineIds: [],
            waitOnParentSuccessPipelineIds: []
        });
        dependencyMode.setIsDependencyModeEnabled(true);
        dependencyMode.setDependencyModeSettings({
            waitOnChildComplete: true,
            waitOnParentComplete: true,
            waitOnParentSuccess: true
        });
        const dependencyModeHandlerChain = new DependencyModeHandlerChain_1.DependencyModeHandlerChain({}, dependencyMode, []);
        const waitOnParentCompleteDependencyModeHandler = jest
            .spyOn(WaitOnParentCompleteDependencyModeHandler_1.WaitOnParentCompleteDependencyModeHandler.prototype, 'setNext')
            .mockReturnThis();
        jest.spyOn(WaitOnChildCompleteDependencyModeHandler_1.WaitOnChildCompleteDependencyModeHandler.prototype, 'setNext').mockReturnThis();
        jest.spyOn(WaitOnParentSuccessDependencyModeHandler_1.WaitOnParentSuccessDependencyModeHandler.prototype, 'setNext').mockReturnThis();
        dependencyModeHandlerChain.handle(context);
        expect(waitOnParentCompleteDependencyModeHandler).toHaveBeenCalledTimes(2);
    });
});
//# sourceMappingURL=dependencyModeComputerChain.test.js.map