"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const InputStep_1 = require("../../inputStep/InputStep");
const RunAdjacencyContext_1 = require("../../RunAdjacencyContext");
const DependencyMode_1 = require("../DependencyMode");
const WaitOnParentSuccessDependencyModeHandler_1 = require("../WaitOnParentSuccessDependencyModeHandler");
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
describe('WaitOnParentSuccessDependencyModeRunHandler init', () => {
    it('should be defined', () => {
        const handler = new WaitOnParentSuccessDependencyModeHandler_1.WaitOnParentSuccessDependencyModeHandler({}, {}, []);
        expect(handler).toBeDefined();
    });
});
describe('WaitOnParentSuccessDependencyModeRunHandler handle', () => {
    const steps = [
        {
            id: 1,
            affinityGroup: 'one'
        },
        {
            id: 2,
            affinityGroup: 'two'
        }
    ];
    const steplets = [
        {
            id: 1,
            stepId: 1,
            affinityGroup: 'one'
        },
        {
            id: 2,
            stepId: 2,
            affinityGroup: 'two'
        }
    ];
    it('should handle when flag enabled', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const firstSteps = [
            {
                id: 1,
                affinityGroup: 'one'
            },
            {
                id: 2,
                affinityGroup: 'two'
            }
        ];
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const dependencyMode = new DependencyMode_1.DependencyMode();
        dependencyMode.setDependencyModeSettings({ waitOnParentSuccess: true });
        dependencyMode.setIsDependencyModeEnabled(true);
        dependencyMode.setDependencyModePipelines({
            waitOnChildCompletePipelineIds: [],
            waitOnParentSuccessPipelineIds: [1],
            waitOnParentCompletePipelineIds: []
        });
        const dependencyModeHandler = new WaitOnParentSuccessDependencyModeHandler_1.WaitOnParentSuccessDependencyModeHandler(inputStep, dependencyMode, firstSteps);
        expect(dependencyModeHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        dependencyModeHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(2);
        expect(context.getValue()).toStrictEqual([
            {
                runId: 1,
                dependentEntityId: 1,
                dependentEntityType: 4,
                affinityGroup: 'one',
                stepId: 1,
                stepletId: 1
            },
            {
                runId: 1,
                dependentEntityId: 1,
                dependentEntityType: 4,
                affinityGroup: 'two',
                stepId: 2,
                stepletId: 2
            }
        ]);
    });
    it('should handle when flag disabled', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const firstSteps = [
            {
                id: 1,
                affinityGroup: 'one'
            },
            {
                id: 2,
                affinityGroup: 'two'
            }
        ];
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const dependencyMode = new DependencyMode_1.DependencyMode();
        dependencyMode.setDependencyModeSettings({ waitOnParentSuccess: false });
        dependencyMode.setIsDependencyModeEnabled(true);
        dependencyMode.setDependencyModePipelines({
            waitOnChildCompletePipelineIds: [],
            waitOnParentSuccessPipelineIds: [1],
            waitOnParentCompletePipelineIds: []
        });
        const dependencyModeHandler = new WaitOnParentSuccessDependencyModeHandler_1.WaitOnParentSuccessDependencyModeHandler(inputStep, dependencyMode, firstSteps);
        expect(dependencyModeHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        dependencyModeHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(0);
        expect(context.getValue()).toStrictEqual([]);
    });
    it('should handle when flag enabled but no pipelines', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const firstSteps = [
            {
                id: 1,
                affinityGroup: 'one'
            },
            {
                id: 2,
                affinityGroup: 'two'
            }
        ];
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const dependencyMode = new DependencyMode_1.DependencyMode();
        dependencyMode.setDependencyModeSettings({ waitOnParentSuccess: true });
        dependencyMode.setIsDependencyModeEnabled(true);
        dependencyMode.setDependencyModePipelines({
            waitOnChildCompletePipelineIds: [],
            waitOnParentSuccessPipelineIds: [],
            waitOnParentCompletePipelineIds: []
        });
        const dependencyModeHandler = new WaitOnParentSuccessDependencyModeHandler_1.WaitOnParentSuccessDependencyModeHandler(inputStep, dependencyMode, firstSteps);
        expect(dependencyModeHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        dependencyModeHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(0);
        expect(context.getValue()).toStrictEqual([]);
    });
});
//# sourceMappingURL=waitOnParentSuccessDependencyModeHandler.test.js.map