"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApprovalGatesHandler = void 0;
const logger_1 = require("@jfrog-pipelines/logger");
const AbstractHandler_1 = require("../chain/AbstractHandler");
const Utils_1 = require("../Utils");
class ApprovalGatesHandler extends AbstractHandler_1.AbstractHandler {
    constructor(inputStep) {
        super();
        this.inputStep = inputStep;
        this.logger = logger_1.LoggerFactory.getLogger('ApprovalGatesHandler');
    }
    handle(context) {
        this.logger.debug('Started');
        const approvalGateSteps = this.inputStep.getApprovalGatesEnabledSteps();
        for (const step of approvalGateSteps) {
            const computedRecords = this.compute(context.getBaseRunAdjacency(), step);
            this.logger.trace(`Computed ${computedRecords.length} for ${step.id}`);
            context.addRecords(computedRecords);
        }
        super.handle(context);
    }
    compute(baseRunAdjcencyObject, step) {
        return [
            ...this.getApprovalGatesRecords(baseRunAdjcencyObject, step, 6),
            ...this.getApprovalGatesRecords(baseRunAdjcencyObject, step, 7)
        ];
    }
    getApprovalGatesRecords(baseRunAdjcencyObject, step, entityType) {
        const records = [];
        const affinityGroupSteps = this.inputStep.getStepsByAffinityGroup(step.affinityGroup);
        for (const affinityGroupStep of affinityGroupSteps) {
            const steplets = this.inputStep.getStepletsByStepId(affinityGroupStep.id);
            records.push(...(0, Utils_1.createRunAdjacencyRecords)(baseRunAdjcencyObject, affinityGroupStep, steplets, {
                dependentEntityId: step.id,
                dependentEntityType: entityType
            }));
        }
        return records;
    }
}
exports.ApprovalGatesHandler = ApprovalGatesHandler;
//# sourceMappingURL=ApprovalGatesHandler.js.map