"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InputStep = void 0;
const utility_1 = require("@jfrog-pipelines/utility");
class InputStep {
    constructor(steps, steplets, runResourceVersions, runStepConnections) {
        this.steps = steps;
        this.steplets = steplets;
        this.runResourceVersions = runResourceVersions;
        this.runStepConnections = runStepConnections;
        this.stepsById = (0, utility_1.keyBy)(this.steps, 'id');
        this.stepletsByStepId = (0, utility_1.groupBy)(this.steplets, 'stepId');
        this.stepsByAffinityGroup = (0, utility_1.groupBy)(this.steps, 'affinityGroup');
        this.runResourcesById = (0, utility_1.keyBy)(this.runResourceVersions, 'id');
        this.runStepConnectionsByStepId = (0, utility_1.groupBy)(this.runStepConnections, 'stepId');
    }
    getStepletsByStepId(stepId) {
        return this.stepletsByStepId[stepId] ?? [];
    }
    getStepsByAffinityGroup(affinityGroup) {
        return this.stepsByAffinityGroup[affinityGroup] ?? [];
    }
    getStepById(stepId) {
        return this.stepsById[stepId];
    }
    getApprovalGatesEnabledSteps() {
        return this.steps.filter((step) => {
            if ((0, utility_1.isEmpty)(step.configPropertyBag)) {
                return false;
            }
            if (typeof step.configPropertyBag.requiresApproval === 'boolean') {
                return step.configPropertyBag.requiresApproval;
            }
            if (typeof step.configPropertyBag.requiresApproval === 'object') {
                return true;
            }
            return false;
        });
    }
    getFirstSteps(inputStepMap) {
        const stepIds = (0, utility_1.keys)(this.stepsById);
        const firstSteps = [];
        for (const stepId of stepIds) {
            const inputStepIds = inputStepMap[Number.parseInt(stepId, 10)];
            if (!inputStepIds || inputStepIds.length === 0) {
                const inputStep = this.stepsById[stepId];
                firstSteps.push(inputStep);
            }
        }
        return firstSteps;
    }
    getSteps() {
        return this.steps;
    }
    getRunResourceVersions() {
        return this.runResourceVersions;
    }
    getRunResourceVersionById(id) {
        return this.runResourcesById[id];
    }
    getRunStepConnectionsByStepId(stepId) {
        return this.runStepConnectionsByStepId[stepId];
    }
}
exports.InputStep = InputStep;
//# sourceMappingURL=InputStep.js.map