"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InputStepHandler = exports.NULL_IDENTIFIER = void 0;
const logger_1 = require("@jfrog-pipelines/logger");
const AbstractHandler_1 = require("../chain/AbstractHandler");
const Utils_1 = require("../Utils");
exports.NULL_IDENTIFIER = -1;
class InputStepHandler extends AbstractHandler_1.AbstractHandler {
    constructor(step, inputStepMap) {
        super();
        this.step = step;
        this.inputStepMap = inputStepMap;
        this.logger = logger_1.LoggerFactory.getLogger('InputStepHandler');
    }
    handle(context) {
        this.logger.debug('Started');
        for (const step of this.step.getSteps()) {
            const computedRecords = this.compute(step, context.getBaseRunAdjacency());
            this.logger.trace(`Computed input step records:  ${computedRecords.length} for step: ${step.id}`);
            context.addRecords(computedRecords);
        }
        super.handle(context);
    }
    compute(step, baseRunAdjacencyObject) {
        const runAdjacencyRecords = [];
        const steplets = this.step.getStepletsByStepId(step.id);
        runAdjacencyRecords.push(...(0, Utils_1.createRunAdjacencyRecords)(baseRunAdjacencyObject, step, steplets, {
            dependentEntityId: exports.NULL_IDENTIFIER,
            dependentEntityType: exports.NULL_IDENTIFIER
        }));
        const inputSteps = this.getInputStepsByStepId(step.id);
        const affinityGroupSteps = this.step.getStepsByAffinityGroup(step.affinityGroup);
        for (const inputStep of inputSteps) {
            let records = [];
            records =
                inputStep.affinityGroup === step.affinityGroup
                    ? this.createStepRecords(step, inputStep, baseRunAdjacencyObject)
                    : this.createAffinityGroupRecords(affinityGroupSteps, step, inputStep, baseRunAdjacencyObject);
            runAdjacencyRecords.push(...records);
        }
        return runAdjacencyRecords;
    }
    getInputStepsByStepId(stepId) {
        const inputStepIds = this.inputStepMap[stepId] ?? [];
        return inputStepIds.map((inputStepId) => {
            return this.step.getStepById(inputStepId);
        });
    }
    createStepRecords(step, inputStep, baseRunAdjacencyObject) {
        const steplets = this.step.getStepletsByStepId(step.id);
        const records = (0, Utils_1.createRunAdjacencyRecords)(baseRunAdjacencyObject, step, steplets, {
            dependentEntityId: inputStep.id,
            dependentEntityType: 0
        });
        return records;
    }
    createAffinityGroupRecords(affinityGroupSteps, step, inputStep, baseRunAdjcencyObject) {
        const records = [];
        for (const affinityGroupStep of affinityGroupSteps) {
            const steplets = this.step.getStepletsByStepId(affinityGroupStep.id);
            records.push(...(0, Utils_1.createRunAdjacencyRecords)(baseRunAdjcencyObject, affinityGroupStep, steplets, {
                dependentEntityId: inputStep.id,
                dependentEntityType: 1
            }));
        }
        return records;
    }
}
exports.InputStepHandler = InputStepHandler;
//# sourceMappingURL=InputStepHandler.js.map