"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const InputStep_1 = require("../InputStep");
const RunAdjacencyContext_1 = require("../../RunAdjacencyContext");
const ApprovalGatesHandler_1 = require("../ApprovalGatesHandler");
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
describe('ApprovalGatesHandler init', () => {
    it('should be defined', () => {
        const approvalGatesHandler = new ApprovalGatesHandler_1.ApprovalGatesHandler({});
        expect(approvalGatesHandler).toBeDefined();
    });
});
describe('ApprovalGatesHandler handle', () => {
    const approvalGateSteps = [
        {
            id: 2,
            affinityGroup: 'two',
            configPropertyBag: {
                requiresApproval: true
            }
        }
    ];
    const steps = [
        {
            id: 2,
            affinityGroup: 'two'
        }
    ];
    const steplets = [
        {
            id: 2,
            stepId: 2,
            affinityGroup: 'two'
        },
        {
            id: 3,
            stepId: 2,
            affinityGroup: 'two'
        }
    ];
    it('should handle when there are approval gates steps', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const inputStep = new InputStep_1.InputStep(approvalGateSteps, steplets, [], []);
        const approvalGatesHandler = new ApprovalGatesHandler_1.ApprovalGatesHandler(inputStep);
        expect(approvalGatesHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        approvalGatesHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(1);
        expect(context.getValue()).toStrictEqual([
            { affinityGroup: 'two', dependentEntityId: 2, dependentEntityType: 6, runId: 1, stepId: 2, stepletId: 2 },
            { affinityGroup: 'two', dependentEntityId: 2, dependentEntityType: 6, runId: 1, stepId: 2, stepletId: 3 },
            { affinityGroup: 'two', dependentEntityId: 2, dependentEntityType: 7, runId: 1, stepId: 2, stepletId: 2 },
            { affinityGroup: 'two', dependentEntityId: 2, dependentEntityType: 7, runId: 1, stepId: 2, stepletId: 3 }
        ]);
    });
    it('should handle when are no approval gates steps', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const approvalGatesHandler = new ApprovalGatesHandler_1.ApprovalGatesHandler(inputStep);
        expect(approvalGatesHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        approvalGatesHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(0);
    });
});
//# sourceMappingURL=ApprovalGatesHandler.test.js.map