"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const InputStep_1 = require("../../inputStep/InputStep");
const RunAdjacencyContext_1 = require("../../RunAdjacencyContext");
const InputStepHandler_1 = require("../InputStepHandler");
jest.mock('@jfrog-pipelines/logger', () => {
    return {
        ...jest.requireActual('@jfrog-pipelines/logger'),
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
describe('InputStepHandler init', () => {
    it('should be defined', () => {
        const approvalGatesHandler = new InputStepHandler_1.InputStepHandler({}, {});
        expect(approvalGatesHandler).toBeDefined();
    });
});
describe('InputStepHandler handle', () => {
    const steps = [
        {
            id: 1,
            affinityGroup: 'one'
        },
        {
            id: 2,
            affinityGroup: 'one'
        },
        {
            id: 3,
            affinityGroup: 'three'
        },
        {
            id: 4,
            affinityGroup: 'three'
        }
    ];
    const steplets = [
        {
            id: 1,
            stepId: 1,
            affinityGroup: 'one'
        },
        {
            id: 2,
            stepId: 2,
            affinityGroup: 'one'
        },
        {
            id: 3,
            stepId: 3,
            affinityGroup: 'three'
        },
        {
            id: 4,
            stepId: 4,
            affinityGroup: 'three'
        }
    ];
    const inputStepMap = {
        1: [3],
        2: [4],
        3: [],
        4: []
    };
    it('should handle when there are step maps', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const inputStep = new InputStep_1.InputStep(steps, steplets, [], []);
        const approvalGatesHandler = new InputStepHandler_1.InputStepHandler(inputStep, inputStepMap);
        expect(approvalGatesHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        approvalGatesHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(4);
        expect(context.getValue()).toStrictEqual([
            { affinityGroup: 'one', dependentEntityId: -1, dependentEntityType: -1, runId: 1, stepId: 1, stepletId: 1 },
            { affinityGroup: 'one', dependentEntityId: 3, dependentEntityType: 1, runId: 1, stepId: 1, stepletId: 1 },
            { affinityGroup: 'one', dependentEntityId: 3, dependentEntityType: 1, runId: 1, stepId: 2, stepletId: 2 },
            { affinityGroup: 'one', dependentEntityId: -1, dependentEntityType: -1, runId: 1, stepId: 2, stepletId: 2 },
            { affinityGroup: 'one', dependentEntityId: 4, dependentEntityType: 1, runId: 1, stepId: 1, stepletId: 1 },
            { affinityGroup: 'one', dependentEntityId: 4, dependentEntityType: 1, runId: 1, stepId: 2, stepletId: 2 },
            { affinityGroup: 'three', dependentEntityId: -1, dependentEntityType: -1, runId: 1, stepId: 3, stepletId: 3 },
            { affinityGroup: 'three', dependentEntityId: -1, dependentEntityType: -1, runId: 1, stepId: 4, stepletId: 4 }
        ]);
    });
    it('should handle when no steps', () => {
        const context = new RunAdjacencyContext_1.RunAdjacencyContext({ runId: 1 });
        const inputStep = new InputStep_1.InputStep([], [], [], []);
        const approvalGatesHandler = new InputStepHandler_1.InputStepHandler(inputStep, {});
        expect(approvalGatesHandler).toBeDefined();
        const addRecordsSpy = jest.spyOn(context, 'addRecords');
        approvalGatesHandler.handle(context);
        expect(addRecordsSpy).toHaveBeenCalledTimes(0);
    });
});
//# sourceMappingURL=InputStepHandler.test.js.map